(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, DB, DBClient, Grids, DBGrids, ExtCtrls, DBCtrls, Provider,
  FireDAC.Comp.Client,
  FireDAC.Comp.DataSet,
  FireDAC.Comp.UI,
  FireDAC.DApt,
  FireDAC.DApt.Intf,
  FireDAC.DatS,
  FireDAC.Phys,
  FireDAC.Phys.IB,
  FireDAC.Phys.IBBase,
  FireDAC.Phys.Intf,
  FireDAC.Stan.Async,
  FireDAC.Stan.Def,
  FireDAC.Stan.Error,
  FireDAC.Stan.Intf,
  FireDAC.Stan.Option,
  FireDAC.Stan.Param,
  FireDAC.Stan.Pool,
  FireDAC.UI.Intf,
  {$IF CompilerVersion < 28}
  // nothing needed
  {$ELSE}
  FireDAC.Phys.IBDef,
  {$ENDIF}
  FireDAC.VCLUI.Wait;

type
  TForm1 = class(TForm)
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    PopupMenu1: TPopupMenu;
    Aboutthiscustomer1: TMenuItem;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    VisitwwwDephiDevelperDayscom1: TMenuItem;
    VisitwwwJensenDataSystemscom1: TMenuItem;
    N2: TMenuItem;
    About1: TMenuItem;
    Create1: TMenuItem;
    NewAggregateField1: TMenuItem;
    SalesTable: TFDQuery;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    SalesTableCustomerTotal: TAggregateField;
    procedure Aboutthiscustomer1Click(Sender: TObject);
    procedure SalesTableCustomerTotalGetText(Sender: TField;
      var Text: String; DisplayText: Boolean);
    procedure About1Click(Sender: TObject);
    procedure VisitwwwDephiDevelperDayscom1Click(Sender: TObject);
    procedure VisitwwwJensenDataSystemscom1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure SalesTableAfterOpen(DataSet: TDataSet);
    procedure FDQuery1CustNoGetText(Sender: TField; var Text: String;
      DisplayText: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure NewAggregateField1Click(Sender: TObject);
    procedure FDQueryt1GetNumberSales(Sender: TField; var Text: String;
      DisplayText: Boolean);
    procedure CustNoGetText(Sender: TField; var Text: String;
      DisplayText: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses ShellApi, SharedDMVclU; //for ShellExecute

procedure TForm1.Aboutthiscustomer1Click(Sender: TObject);
begin
ShowMessage('The average sale to this customer is ' +
  Format('%.2m', [StrToFloat(SalesTable.Aggregates[0].Value)]) +
  '. The first sale to this customer was on '+
  DateToStr(SalesTable.Aggregates[1].Value));
end;

procedure TForm1.SalesTableCustomerTotalGetText(Sender: TField;
  var Text: String; DisplayText: Boolean);
begin
  if gbLast in SalesTable.GetGroupState(1) then
    Text := Format('%.2m',[StrToFloat(Sender.Value)])
  else
    Text := '';
end;

procedure TForm1.FDQueryt1GetNumberSales(Sender: TField;
  var Text: String; DisplayText: Boolean);
begin
  if gbLast in SalesTable.GetGroupState(1) then
    Text := Sender.Value
  else
    Text := '';
end;

procedure TForm1.CustNoGetText(Sender: TField;
  var Text: String; DisplayText: Boolean);
begin
  if gbFirst in SalesTable.GetGroupState(1) then
    Text := Sender.Value
  else
    Text := '';
end;



procedure TForm1.FDQuery1CustNoGetText(Sender: TField;
  var Text: String; DisplayText: Boolean);
begin
  if gbFirst in SalesTable.GetGroupState(1) then
    Text := Sender.Value
  else
    Text := '';
end;

procedure TForm1.SalesTableAfterOpen(DataSet: TDataSet);
begin
  //Hook up the OnGetText event handler
  SalesTable.FieldByName('CUST_NO').OnGetText :=
    FDQuery1CustNoGetText;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
ShowMessage('FDAggregatesAndGroupState (c) 2017 Cary Jensen, Jensen Data Systems, Inc. '
  + #10#13 + ''
  + #10#13 + 'Demonstration of how to use aggregate fields, aggregates, and group state.'
  + #10#13 + 'This project is provided for demonstration purposes only.'
  + #10#13 + ''
  + #10#13 + 'Visit http://www.JensenDataSystems.com for information about training and consulting services.');
end;

procedure TForm1.VisitwwwDephiDevelperDayscom1Click(Sender: TObject);
begin
ShellExecute(Application.Handle, 'open',
 'http://www.DelphiDeveloperDays.com', nil, nil, SW_SHOW);
end;

procedure TForm1.VisitwwwJensenDataSystemscom1Click(Sender: TObject);
begin
ShellExecute(Application.Handle, 'open',
 'http://www.JensenDataSystems.com', nil, nil, SW_SHOW);
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
Self.Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  SalesTable.Open();
  SalesTable.FieldByName('CUST_NO').OnGetText := CustNoGetText;
end;

procedure TForm1.NewAggregateField1Click(Sender: TObject);
begin
  SalesTable.DisableControls;
  SalesTable.Close;
  try
    with TAggregateField.Create(Self) do
    begin
      FieldName := 'NumberOfSales';
      Expression := 'COUNT(CUST_NO)';
      IndexName := 'CustIdx';
      GroupingLevel := 1;
      Active := True;
      Visible := True;
      Name := 'FDQuery1NumberOfSales';
      DataSet := SalesTable;
    end;
    with DBGrid1.Columns.Add do
    begin
      FieldName := 'NumberOfSales';
      Title.Caption := 'Number of Sales';
      Index := 5;
     end;
    //Hook up the OnGetText event handler
    SalesTable.FieldByName('NumberOfSales').OnGetText :=
      FDQueryt1GetNumberSales;
    //Disable the menu to that creates the new AggregateField
    Create1.Enabled := False;
  finally
    //Enable controls and re-open the ClientDataSet
    SalesTable.EnableControls;
    SalesTable.Open;
    //Hook up the CUSTNO OnGetText event handler
    SalesTable.FieldByName('CUST_NO').OnGetText :=
      FDQuery1CustNoGetText;
  end;
end;

end.
