(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, Data.DB, Vcl.ComCtrls, Vcl.Grids, Vcl.DBGrids,
  Vcl.ExtCtrls, Vcl.DBCtrls, FireDAC.Comp.DataSet, FireDAC.Comp.Client,
  FireDAC.Stan.StorageBin, Vcl.StdCtrls, FireDAC.DApt, FireDAC.Stan.Async;

type
  TForm1 = class(TForm)
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    StatusBar1: TStatusBar;
    DataSource1: TDataSource;
    btnShowBackup: TButton;
    FDMemTable1: TFDMemTable;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnShowBackupClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses SharedDMVclU,
     BackupFormU,
     FireDAC.Stan.Util; // for FDExpandStr

procedure TForm1.btnShowBackupClick(Sender: TObject);
var
  BackupForm: TBackupForm;
  BackupFile: String;
begin
  BackupFile := FDManager.ResourceOptions.BackupFolder  +
                  ChangeFileExt( FDMemTable1.ResourceOptions.PersistentFileName,
                  FDManager.ResourceOptions.BackupExt );
  if not FileExists( FDExpandStr( BackupFile) ) then
    raise Exception.Create('A backup file has not been created');
  BackupForm := TBackupForm.Create( nil );
  try
    BackupForm.FDMemTable1.LoadFromFile( BackupFile, TFDStorageFormat.sfXML );
    BackupForm.ShowModal;
  finally
    BackupForm.Release;
  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if FDMemTable1.State in dsEditModes then
    FDMemTable1.Post;
  FDMemTable1.Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  BackupFolder: string;
begin
  FDManager.ResourceOptions.BackupFolder := '$(RUN)\Backup\';
  FDManager.ResourceOptions.BackupExt := '.BAK';
  BackupFolder := FDExpandStr(FDManager.ResourceOptions.BackupFolder);
  if not DirectoryExists( BackupFolder ) then
    ForceDirectories( ExtractFilePath( BackupFolder ) );
  FDManager.ResourceOptions.DefaultStoreFolder := '$(RUN)';
  FDManager.ResourceOptions.DefaultStoreFormat := sfXML;
  FDMemTable1.ResourceOptions.Backup := True;
  FDMemTable1.ResourceOptions.PersistentFileName := 'Clients.xml';
  FDMemTable1.ResourceOptions.Persistent := True;
  if not FileExists( FDMemTable1.ResourceOptions.PersistentFileName ) then
  begin
    with FDMemTable1.FieldDefs do
    begin
      Clear;
      Add('ClientID', ftInteger);
      Add('ClientName', ftString, 45 );
      Add('Address1', ftString, 50 );
      Add('Address2', ftString, 50 );
      Add('LastContact', ftDateTime );
      Add('Notes', ftMemo );
      Add('Active',ftBoolean);
    end; //with FDMemTable.FieldDefs
    FDMemTable1.CreateDataSet;
  end
  else
    FDMemTable1.Open;
end;

end.
