(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Error, FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def,
  FireDAC.Stan.Pool, FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Phys.IB,
  FireDAC.VCLUI.Wait, FireDAC.Stan.Param, FireDAC.DatS,
  FireDAC.DApt.Intf, FireDAC.DApt, Data.DB, Vcl.StdCtrls, Vcl.Grids,
  Vcl.DBGrids, FireDAC.Comp.DataSet, FireDAC.Comp.Client, Vcl.ComCtrls,
  Vcl.Samples.Spin, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    EmployeeConnection: TFDConnection;
    TwoFieldsTable: TFDQuery;
    DBGrid1: TDBGrid;
    btnCancelUpdates: TButton;
    DataSource1: TDataSource;
    btnApplyUpdates: TButton;
    btnCancelUpdatesAndEmptyTwoFieldsTable: TButton;
    Label1: TLabel;
    Label2: TLabel;
    CurrentTwoFieldsTable: TFDQuery;
    DataSource2: TDataSource;
    DBGrid2: TDBGrid;
    StatusBar1: TStatusBar;
    Label3: TLabel;
    TwoFieldsTableFIRST: TStringField;
    TwoFieldsTableLAST: TStringField;
    TwoFieldsTableCacheStatus: TStringField;
    cbxAutoCommitUpdates: TCheckBox;
    sedMaxErrors: TSpinEdit;
    rgpHandleErrors: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure btnApplyUpdatesClick(Sender: TObject);
    procedure TwoFieldsTableUpdateError(ASender: TDataSet; AException: EFDException;
      ARow: TFDDatSRow; ARequest: TFDUpdateRequest;
      var AAction: TFDErrorAction);
    procedure TwoFieldsTableReconcileError(DataSet: TFDDataSet; E: EFDException;
      UpdateKind: TFDDatSRowState; var Action: TFDDAptReconcileAction);
    procedure btnCancelUpdatesAndEmptyTwoFieldsTableClick(Sender: TObject);
    procedure TwoFieldsTableCalcFields(DataSet: TDataSet);
    procedure btnCancelUpdatesClick(Sender: TObject);
    procedure TwoFieldsTableUpdateRecord(ASender: TDataSet;
      ARequest: TFDUpdateRequest; var AAction: TFDErrorAction;
      AOptions: TFDUpdateRowOptions);
    procedure cbxAutoCommitUpdatesClick(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure rgpHandleErrorsClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.btnApplyUpdatesClick(Sender: TObject);
var
  NumErrors: Integer;
begin
  if TwoFieldsTable.State in dsEditModes then
    TwoFieldsTable.Post;
  NumErrors := TwoFieldsTable.ApplyUpdates( sedMaxErrors.Value );
  TwoFieldsTable.FilterChanges := [ rtModified, rtInserted, rtDeleted,
                                    rtHasErrors ];
  CurrentTwoFieldsTable.Refresh;
  StatusBar1.SimpleText := 'There were ' + NumErrors.ToString + ' errors ' +
    'encountered. Records remaining in cache: ' +
    TwoFieldsTable.ChangeCount.ToString;
end;

procedure TForm1.btnCancelUpdatesClick(Sender: TObject);
begin
  TwoFieldsTable.FilterChanges := [ rtUnmodified, rtModified, rtInserted,
                                    rtDeleted, rtHasErrors ];
  TwoFieldsTable.CancelUpdates;
  TwoFieldsTable.Refresh;
  CurrentTwoFieldsTable.Refresh;
  StatusBar1.SimpleText := 'The cache was canceled';
end;

procedure TForm1.cbxAutoCommitUpdatesClick(Sender: TObject);
begin
  {$IF CompilerVersion >= 30.0}
  TwoFieldsTable.UpdateOptions.AutoCommitUpdates := cbxAutoCommitUpdates.Checked;
  {$ENDIF}
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  if TwoFieldsTable.RowError <> nil then
    StatusBar1.SimpleText := 'RowError: ' + TwoFieldsTable.RowError.Message
  else
    StatusBar1.SimpleText := '';
end;

procedure TForm1.btnCancelUpdatesAndEmptyTwoFieldsTableClick(Sender: TObject);
begin
  TwoFieldsTable.CancelUpdates;
  EmployeeConnection.StartTransaction;
  try
    EmployeeConnection.ExecSQL('DELETE FROM TWOFIELDSTABLE');
    EmployeeConnection.Commit;
  except
    EmployeeConnection.Rollback;
    raise;
  end;
  TwoFieldsTable.FilterChanges := [ rtUnmodified, rtModified, rtInserted,
                                    rtDeleted, rtHasErrors ];
  TwoFieldsTable.Refresh;
  CurrentTwoFieldsTable.Refresh;
  StatusBar1.SimpleText := 'The cache was canceled and TwoFieldsTable was emptied';
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  FDQuery: TFDQuery;
const
  TabName = 'TWOFIELDSTABLE';
begin
  EmployeeConnection.Open();
  FDQuery := TFDQuery.Create( nil );
  try
    FDQuery.Connection := EmployeeConnection;
    FDQuery.SQL.Text := 'SELECT RDB$RELATION_NAME FROM RDB$RELATIONS ' +
                         '  WHERE RDB$RELATION_NAME = ''' + TabName + ''';';
    FDQuery.Open();
    if FDQuery.RecordCount = 0 then
    begin
      EmployeeConnection.StartTransaction;
      try
      FDQuery.ExecSQL('CREATE TABLE "'+ TabName + '" (' +
                                   '"FIRST"	VARCHAR(30) NOT NULL, ' +
                                   '"LAST"	VARCHAR(30) NOT NULL );');
        EmployeeConnection.Commit;
      except
        EmployeeConnection.Rollback;
        raise Exception.Create('Cannot create ' + TabName);
      end;
    end;
    FDQuery.Close;
  finally
    FDQuery.Free;
  end;
  TwoFieldsTable.SQL.Text := 'SELECT * FROM ' + TabName;
  {$IF CompilerVersion >= 30.0}  // Delphi 10 Seattle or later
  TwoFieldsTable.UpdateOptions.AutoCommitUpdates := True;
  {$ENDIF}
  TwoFieldsTable.UpdateOptions.CheckRequired := False;
  TwoFieldsTable.UpdateOptions.CheckReadOnly := False;
  TwoFieldsTable.UpdateOptions.CheckUpdatable := False;
  TwoFieldsTable.CachedUpdates := True;
  TwoFieldsTable.Open;
  {$IF CompilerVersion < 30}
    cbxAutoCommitUpdates.Enabled := False;
  {$ELSE} // Delphi 10 Seattle or later
    TwoFieldsTable.UpdateOptions.AutoCommitUpdates :=
      cbxAutoCommitUpdates.Checked;
  {$ENDIF}
  CurrentTwoFieldsTable.Open('SELECT * FROM ' + TabName );
end;

procedure TForm1.rgpHandleErrorsClick(Sender: TObject);
begin
  case rgpHandleErrors.ItemIndex of
    0:
    begin
      TwoFieldsTable.OnUpdateError := nil;
      TwoFieldsTable.OnReconcileError := nil;
      TwoFieldsTable.UpdateOptions.UpdateChangedFields := True;
    end;
    1:
    begin
      TwoFieldsTable.OnUpdateError := TwoFieldsTableUpdateError;
      TwoFieldsTable.UpdateOptions.UpdateChangedFields := False;
      TwoFieldsTable.OnReconcileError := nil;
    end;
    2:
    begin
      TwoFieldsTable.OnUpdateError := nil;
      TwoFieldsTable.OnReconcileError := TwoFieldsTableReconcileError;
      TwoFieldsTable.UpdateOptions.UpdateChangedFields := True;
    end;
  end;
end;

procedure TForm1.TwoFieldsTableCalcFields(DataSet: TDataSet);
begin
  case DataSet.UpdateStatus of
   usUnmodified: DataSet.FieldByName('CacheStatus').AsString := 'Unmodified';
     usModified: DataSet.FieldByName('CacheStatus').AsString := 'Modified';
     usInserted: DataSet.FieldByName('CacheStatus').AsString := 'Inserted';
      usDeleted: DataSet.FieldByName('CacheStatus').AsString := 'Deleted';
  end;
end;

procedure TForm1.TwoFieldsTableReconcileError(DataSet: TFDDataSet; E: EFDException;
  UpdateKind: TFDDatSRowState; var Action: TFDDAptReconcileAction);
begin
  if  (not DataSet.Fields[0].IsNull) and (not DataSet.Fields[1].IsNull) then
    Action := raSkip
  else
  if DataSet.Fields[0].IsNull and DataSet.Fields[1].IsNull then
    Action := raSkip
  else
  if DataSet.Fields[0].IsNull then
  begin
    DataSet.Edit;
    DataSet.Fields[0].Value := DataSet.Fields[1].Value;
    DataSet.Post;
    Action := raCorrect;
  end
  else
  if DataSet.Fields[1].IsNull then
  begin
    DataSet.Edit;
    DataSet.Fields[1].Value := DataSet.Fields[0].Value;
    DataSet.Post;
    Action := raCorrect;
  end
  else
    Action := raSkip;
end;

procedure TForm1.TwoFieldsTableUpdateError(ASender: TDataSet;
  AException: EFDException; ARow: TFDDatSRow; ARequest: TFDUpdateRequest;
  var AAction: TFDErrorAction);
begin
  if  (not ASender.Fields[0].IsNull) and (not ASender.Fields[1].IsNull) then
    AAction := eaDefault
  else
  if ASender.Fields[0].IsNull and ASender.Fields[1].IsNull then
    AAction := eaFail
  else
  if ASender.Fields[0].IsNull then
  begin
    ASender.Edit;
    ASender.Fields[0].Value := ASender.Fields[1].Value;
    ASender.Post;
    AAction := eaRetry;
  end
  else
  if ASender.Fields[1].IsNull then
  begin
    ASender.Edit;
    ASender.Fields[1].Value := ASender.Fields[0].Value;
    ASender.Post;
    AAction := eaRetry;
  end
  else
    AAction := eaFail;
end;

procedure TForm1.TwoFieldsTableUpdateRecord(ASender: TDataSet;
  ARequest: TFDUpdateRequest; var AAction: TFDErrorAction;
  AOptions: TFDUpdateRowOptions);
var
  FDQuery: TFDQuery;
begin
  AAction := eaDefault;exit;
  AAction := eaFail;
  FDQuery := TFDQuery.Create( nil );
  try
    FDQuery.Connection := EmployeeConnection;
    case ARequest of
      arInsert:
        begin
          FDQuery.SQL.Text := 'INSERT INTO TwoFieldsTable ("FIRST", "LAST")' +
                              '  VALUES ( :f, :l );';
          FDQuery.Params[0].AsString := ASender.FieldByName('FIRST').CurValue;
          FDQuery.Params[1].AsString := ASender.FieldByName('LAST').CurValue;
          FDQuery.ExecSQL;
          AAction := eaApplied;
        end;
      arUpdate:
        begin
          FDQuery.SQL.Text := 'UPDATE TwoFieldsTable ' +
                              '  SET "FIRST" = :cf, ' +
                              '      "LAST" = :cl ' +
                              '  WHERE "FIRST" = :of ' +
                              '    AND "LAST" = :ol;';
          FDQuery.Params[0].AsString := ASender.FieldByName('FIRST').CurValue;
          FDQuery.Params[1].AsString := ASender.FieldByName('LAST').CurValue;
          FDQuery.Params[2].AsString := ASender.FieldByName('FIRST').OldValue;
          FDQuery.Params[3].AsString := ASender.FieldByName('LAST').OldValue;
          FDQuery.ExecSQL;
          AAction := eaApplied;
        end;
      arDelete:
        begin
          FDQuery.SQL.Text := 'DELETE FROM TwoFieldsTable ' +
                              '  WHERE "FIRST" = :f ' +
                              '    AND "LAST" = :l;';
          FDQuery.Params[0].AsString := ASender.FieldByName('FIRST').OldValue;
          FDQuery.Params[1].AsString := ASender.FieldByName('LAST').OldValue;
          FDQuery.ExecSQL;
          AAction := eaApplied;
        end;
      else
      begin
        AAction := eaDefault;
      end;
    end;
  finally
    FDQuery.Free;
  end;
end;

end.
