(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, Grids, DBGrids, ExtCtrls, DBCtrls, DBClient, FireDAC.Stan.Intf,
  FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS,
  FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt,
  FireDAC.UI.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Phys,
  FireDAC.VCLUI.Wait, FireDAC.Comp.Client, FireDAC.Comp.DataSet,
  FireDAC.Phys.IB, Vcl.StdCtrls,
  FireDAC.Stan.ExprFuncs;  // This unit is critical if you want to use FireDAC functions


type
  TForm1 = class(TForm)
    DataSource1: TDataSource;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    FDQuery1: TFDQuery;
    FDQuery1Total: TCurrencyField;
    FDQuery1ORDERNO: TFloatField;
    FDQuery1ITEMNO: TFloatField;
    FDQuery1PARTNO: TFloatField;
    FDQuery1QTY: TIntegerField;
    FDQuery1DISCOUNT: TFloatField;
    FDQuery1LISTPRICE: TFloatField;
    Label1: TLabel;
    FDQuery1NewTotal: TFloatField;
    procedure FormCreate(Sender: TObject);
    procedure FDQuery1CalcFields(DataSet: TDataSet);
    procedure DBGrid1TitleClick(Column: TColumn);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses DataPaths, SharedDMVclU;

function SortFDDataSetWithFieldName(DataSet: TFDDataSet;
  const FieldName: String): Boolean;
var
  Field: TField;
begin
  Result := False;
  Field := DataSet.Fields.FindField(FieldName);
  if not Assigned(Field) then
    Exit;

  // if field type is not sortable, exit.
  if (Field is TObjectField) or (Field is TBlobField)
    or (Field is TVariantField) or (Field is TBinaryField)
    or not (Field.FieldKind in [ fkData, fkInternalCalc ]) then
    exit;

  if SameText(DataSet.IndexFieldNames, FieldName) or
    SameText(DataSet.IndexFieldNames, FieldName + ':A') then
    DataSet.IndexFieldNames := FieldName + ':D'
  else
    DataSet.IndexFieldNames := FieldName + ':A';

  Result := True;
end;

procedure TForm1.DBGrid1TitleClick(Column: TColumn);
begin
  SortFDDataSetWithFieldName( TFDDataSet(Column.Field.DataSet), Column.FieldName);
end;

procedure TForm1.FDQuery1CalcFields(DataSet: TDataSet);
begin
 FDQuery1Total.AsCurrency :=
   FDQuery1LISTPRICE.AsCurrency * FDQuery1QTY.AsInteger -
    (FDQuery1LISTPRICE.AsCurrency * FDQuery1QTY.AsInteger * (FDQuery1DISCOUNT.AsInteger / 100));

  //If there were additional InternalCalc or Calculated fields in
  //this dataset, you would perform those calculations here.
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Connection: TFDConnection;
begin
  Connection := SharedDMVcl.FDConnection;
  Connection.Params.Clear;
  Connection.Params.Add('User_Name=sysdba');
  Connection.Params.Add('Password=masterkey');
  Connection.Params.Add('Protocol=TCPIP');
  Connection.Params.Add('DriverID=IB');
  Connection.Params.Add('Database=' + DataPaths.SamplePath + '\dbdemos.gdb');
  Connection.Open();
  FDQuery1.Connection := Connection;
  FDQuery1.Open();
//  FDQuery1.OnCalcFields := FDQuery1CalcFields;
end;

end.
