(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, Data.DB, Vcl.StdCtrls,
  Vcl.ExtCtrls, Vcl.DBCtrls, Vcl.Grids, Vcl.DBGrids, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client;

type
  TMainform = class(TForm)
    FDQuery1: TFDQuery;
    DBGrid1: TDBGrid;
    DBGrid2: TDBGrid;
    DBNavigator1: TDBNavigator;
    DBNavigator2: TDBNavigator;
    DataSource1: TDataSource;
    DataSource2: TDataSource;
    btnCloneQuery1toMemTable1: TButton;
    FDQuery1UpdateStatus: TStringField;
    FDQuery1EMP_NO: TSmallintField;
    FDQuery1FIRST_NAME: TStringField;
    FDQuery1LAST_NAME: TStringField;
    FDQuery1PHONE_EXT: TStringField;
    FDQuery1HIRE_DATE: TSQLTimeStampField;
    FDQuery1DEPT_NO: TStringField;
    FDQuery1JOB_CODE: TStringField;
    FDQuery1JOB_GRADE: TSmallintField;
    FDQuery1JOB_COUNTRY: TStringField;
    FDQuery1SALARY: TBCDField;
    FDQuery1FULL_NAME: TStringField;
    btnApplyUpdates: TButton;
    cbxCachedUpdates: TCheckBox;
    btnOpenQuery1: TButton;
    btnCloseFDMemTable1: TButton;
    FDMemTable1: TFDMemTable;
    FDMemTable2: TFDMemTable;
    btnCloneQuery1toMemTable2: TButton;
    btnCloseFDMemTable2: TButton;
    DBNavigator3: TDBNavigator;
    DataSource3: TDataSource;
    DBGrid3: TDBGrid;
    procedure FDQuery1CalcFields(DataSet: TDataSet);
    procedure btnCloneQuery1toMemTable1Click(Sender: TObject);
    procedure btnOpenQuery1Click(Sender: TObject);
    procedure btnApplyUpdatesClick(Sender: TObject);
    procedure btnCloseFDMemTable1Click(Sender: TObject);
    procedure btnCloneQuery1toMemTable2Click(Sender: TObject);
    procedure btnCloseFDMemTable2Click(Sender: TObject);
    procedure cbxCachedUpdatesClick(Sender: TObject);
    procedure FDMemTable1AfterClose(DataSet: TDataSet);
    procedure FDMemTable1AfterOpen(DataSet: TDataSet);
    procedure FDMemTable2AfterOpen(DataSet: TDataSet);
    procedure FDMemTable2AfterClose(DataSet: TDataSet);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Mainform: TMainform;

implementation

{$R *.dfm}

uses SharedDMVclU, TypInfo;

procedure TMainform.btnApplyUpdatesClick(Sender: TObject);
begin
  if FDQuery1.Active then
    FDQuery1.ApplyUpdates( 0 )
  else
  if FDMemTable1.Active then
    FDMemTable1.ApplyUpdates( 0 )
  else
  if FDMemTable2.Active then
    FDMemTable2.ApplyUpdates( 0 );
end;

procedure TMainform.btnOpenQuery1Click(Sender: TObject);
begin
  if not FDQuery1.Active then
  begin
    FDQuery1.Open();
    FDQuery1.CachedUpdates := cbxCachedUpdates.Checked;
    btnApplyUpdates.Enabled := cbxCachedUpdates.Checked;
    btnOpenQuery1.Caption := 'Close FDQuery1';
  end
  else
  begin
    if FDQuery1.ChangeCount > 0 then
      FDQuery1.CancelUpdates;
    FDQuery1.Close;
    btnApplyUpdates.Enabled := FDMemTable1.Active;
    btnOpenQuery1.Caption := 'Open FDQuery1';
  end;
end;

procedure TMainform.cbxCachedUpdatesClick(Sender: TObject);
var
  NotifyEvent: TNotifyEvent;
begin
  if FDQuery1.Active then
      if FDQuery1.ChangeCount > 0 then
      begin
        NotifyEvent := cbxCachedUpdates.OnClick;
        cbxCachedUpdates.OnClick := nil;
        cbxCachedUpdates.Checked := True;
        cbxCachedUpdates.OnClick := NotifyEvent;
        raise Exception.Create('Apply changes or close FDQuery1 ' +
                               'before change cached updates' );
      end;
  // ok to change
  FDQuery1.CachedUpdates := cbxCachedUpdates.Checked;
  btnApplyUpdates.Enabled := cbxCachedUpdates.Checked;
end;

procedure TMainform.btnCloneQuery1toMemTable1Click(Sender: TObject);
begin
  FDMemTable1.CloneCursor( FDQuery1 );
end;

procedure TMainform.btnCloneQuery1toMemTable2Click(Sender: TObject);
begin
  FDMemTable2.CloneCursor( FDQuery1 );
  btnCloseFDMemTable2.Enabled := True;
end;

procedure TMainform.btnCloseFDMemTable1Click(Sender: TObject);
begin
  FDMemTable1.Close;
end;

procedure TMainform.btnCloseFDMemTable2Click(Sender: TObject);
begin
  FDMemTable2.Close;
end;

procedure TMainform.FDMemTable1AfterClose(DataSet: TDataSet);
begin
  btnCloseFDMemTable1.Enabled := False;
end;

procedure TMainform.FDMemTable1AfterOpen(DataSet: TDataSet);
begin
  btnCloseFDMemTable1.Enabled := True;
end;

procedure TMainform.FDMemTable2AfterClose(DataSet: TDataSet);
begin
  btnCloseFDMemTable2.Enabled := False;
end;

procedure TMainform.FDMemTable2AfterOpen(DataSet: TDataSet);
begin
  btnCloseFDMemTable2.Enabled := True;
end;

procedure TMainform.FDQuery1CalcFields(DataSet: TDataSet);
begin
  FDQuery1UpdateStatus.AsString := GetEnumName( TypeInfo(TUpdateStatus), Ord(FDQuery1.UpdateStatus) );
end;

end.
