(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, Datasnap.DBClient, Vcl.Grids,
  Vcl.DBGrids, Vcl.ExtCtrls, Vcl.DBCtrls, FireDAC.Stan.Intf,
  FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS,
  FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Stan.StorageBin, Vcl.StdCtrls,
  FireDAC.Comp.DataSet, FireDAC.Comp.Client;

type
  TMainForm = class(TForm)
    ClientDataSet1: TClientDataSet;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    FDMemTable1: TFDMemTable;
    DBGrid2: TDBGrid;
    DataSource2: TDataSource;
    DBNavigator2: TDBNavigator;
    FDMemTable1CustNo: TFloatField;
    FDMemTable1Company: TStringField;
    Button1: TButton;
    ListBox1: TListBox;
    btnClose: TButton;
    cbxStructure: TCheckBox;
    cbxRestart: TCheckBox;
    cbxAppend: TCheckBox;
    cbxEdit: TCheckBox;
    cbxDelete: TCheckBox;
    cbxRefresh: TCheckBox;
    cbxAggregatesReset: TCheckBox;
    cbxAggregatesCopy: TCheckBox;
    cbxConstraintsReset: TCheckBox;
    cbxConstraintsCopy: TCheckBox;
    cbxIndexesReset: TCheckBox;
    cbxIndexesCopy: TCheckBox;
    cbxCalcFields: TCheckBox;
    FDMemTable1Contact: TStringField;
    ClientDataSet1CustNo: TFloatField;
    ClientDataSet1Company: TStringField;
    ClientDataSet1Addr1: TStringField;
    ClientDataSet1Addr2: TStringField;
    ClientDataSet1City: TStringField;
    ClientDataSet1State: TStringField;
    ClientDataSet1Zip: TStringField;
    ClientDataSet1Country: TStringField;
    ClientDataSet1Phone: TStringField;
    ClientDataSet1FAX: TStringField;
    ClientDataSet1TaxRate: TFloatField;
    ClientDataSet1Contact: TStringField;
    ClientDataSet1LastInvoiceDate: TDateTimeField;
    ClientDataSet1UpdateStatus: TStringField;
    FDMemTable1Field1: TStringField;
    btnClearFields: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FDMemTable1AfterDelete(DataSet: TDataSet);
    procedure FDMemTable1AfterInsert(DataSet: TDataSet);
    procedure FDMemTable1AfterPost(DataSet: TDataSet);
    procedure FDMemTable1BeforeDelete(DataSet: TDataSet);
    procedure FDMemTable1BeforeInsert(DataSet: TDataSet);
    procedure FDMemTable1BeforePost(DataSet: TDataSet);
    procedure ClientDataSet1CalcFields(DataSet: TDataSet);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnClearFieldsClick(Sender: TObject);
  private
    function BuildCopyDataSetOptions: TFDCopyDataSetOptions;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses DataPaths, SharedDMVclU, TypInfo;

procedure TMainForm.btnClearFieldsClick(Sender: TObject);
begin
  FDMemTable1.Fields.Clear;
end;

procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  FDMemTable1.Close;
end;

function TMainForm.BuildCopyDataSetOptions: TFDCopyDataSetOptions;
begin
  Result := [];
  if cbxStructure.Checked then Result := Result + [coStructure];
  if cbxRestart.Checked then Result := Result + [coRestart];
  if cbxAppend.Checked then Result := Result + [coAppend];
  if cbxEdit.Checked then Result := Result + [coEdit];
  {$IF CompilerVersion > 27.0}
  if cbxDelete.Checked then Result := Result + [coDelete];
  {$ENDIF}
  if cbxRefresh.Checked then Result := Result + [coRefresh];
  if cbxCalcFields.Checked then Result := Result + [coCalcFields];
  if cbxAggregatesReset.Checked then Result := Result + [coAggregatesReset];
  if cbxAggregatesCopy.Checked then Result := Result + [coAggregatesCopy];
  if cbxConstraintsReset.Checked then Result := Result + [coConstraintsReset];
  if cbxConstraintsCopy.Checked then Result := Result + [coConstraintsCopy];
  if cbxIndexesReset.Checked then Result := Result + [coIndexesReset];
  if cbxIndexesCopy.Checked then Result := Result + [coIndexesCopy];
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  CopyDataSetOptions: TFDCopyDataSetOptions;
begin
  CopyDataSetOptions := BuildCopyDataSetOptions;
  if not (coStructure in CopyDataSetOptions) then
    FDMemTable1.Open;
  FDMemTable1.CopyDataSet( ClientDataSet1, CopyDataSetOptions );
end;

procedure TMainForm.ClientDataSet1CalcFields(DataSet: TDataSet);
begin
  ClientDataSet1UpdateStatus.AsString := GetEnumName( TypeInfo(TUpdateStatus), Ord(ClientDataSet1.UpdateStatus) );
end;

procedure TMainForm.FDMemTable1AfterDelete(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'After delete' );
end;

procedure TMainForm.FDMemTable1AfterInsert(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'After insert' );
end;

procedure TMainForm.FDMemTable1AfterPost(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'After post' );
end;

procedure TMainForm.FDMemTable1BeforeDelete(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'Before delete' );
end;

procedure TMainForm.FDMemTable1BeforeInsert(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'Before insert' );
end;

procedure TMainForm.FDMemTable1BeforePost(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'Before post' );
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // We don't want to save changes to this test data
  ClientDataSet1.CancelUpdates;
  ClientDataSet1.Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  DataPaths.ValidatePath(SamplePath);
  ClientDataSet1.FileName := SamplePath + '\Customer.cds';
  ClientDataSet1.Open;
end;

end.
