(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, Vcl.Grids, Vcl.DBGrids,
  Vcl.StdCtrls, FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Param,
  FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf,
  FireDAC.Stan.Async, FireDAC.DApt, FireDAC.Comp.Client, FireDAC.Comp.DataSet,
  Vcl.ExtCtrls;

type
  TMainform = class(TForm)
    btnCreateSource: TButton;
    btnCreateDestination: TButton;
    btnCopyDataSet: TButton;
    edtSQLSelect: TEdit;
    memCreateSQL: TMemo;
    DBGrid1: TDBGrid;
    cbxStructure: TCheckBox;
    cbxRestart: TCheckBox;
    cbxAppend: TCheckBox;
    cbxEdit: TCheckBox;
    cbxDelete: TCheckBox;
    cbxRefresh: TCheckBox;
    cbxAggregatesReset: TCheckBox;
    cbxAggregatesCopy: TCheckBox;
    cbxConstraintsReset: TCheckBox;
    cbxConstraintsCopy: TCheckBox;
    cbxIndexesReset: TCheckBox;
    cbxIndexesCopy: TCheckBox;
    cbxCalcFields: TCheckBox;
    ListBox1: TListBox;
    Label1: TLabel;
    rgpDestinationType: TRadioGroup;
    DBGrid2: TDBGrid;
    sourceDataSource: TDataSource;
    destinationDataSource: TDataSource;
    Label2: TLabel;
    destTabName: TEdit;
    cbxCachedSourceUpdates: TCheckBox;
    cbxCachedDestUpdates: TCheckBox;
    btnApplyUpdates: TButton;
    procedure AfterDelete(DataSet: TDataSet);
    procedure AfterInsert(DataSet: TDataSet);
    procedure AfterPost(DataSet: TDataSet);
    procedure BeforeDelete(DataSet: TDataSet);
    procedure BeforeInsert(DataSet: TDataSet);
    procedure BeforePost(DataSet: TDataSet);
    procedure btnCreateDestinationClick(Sender: TObject);
    procedure btnCreateSourceClick(Sender: TObject);
    procedure btnCopyDataSetClick(Sender: TObject);
    procedure btnApplyUpdatesClick(Sender: TObject);
  private
    { Private declarations }
  protected
    function BuildCopyDataSetOptions: TFDCopyDataSetOptions;
    procedure WireUpEventHandlers( FDDataSet: TFDDataSet );
    function IsDestTableNameProbably: Boolean;
  public
    { Public declarations }
    srcFDQuery: TFDQuery;
    FDDataSet: TFDDataSet;
  end;

var
  Mainform: TMainform;

implementation

{$R *.dfm}

uses SharedDMVclU;

{ TMainform }

procedure TMainform.AfterDelete(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'DataSet.Name: ' + 'After delete' );
end;

procedure TMainform.AfterInsert(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'DataSet.Name: ' + 'After insert' );
end;

procedure TMainform.AfterPost(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'DataSet.Name: ' + 'After post' );
end;

procedure TMainform.BeforeDelete(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'DataSet.Name: ' + 'Before delete' );
end;

procedure TMainform.BeforeInsert(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'DataSet.Name: ' + 'Before insert' );
end;

procedure TMainform.BeforePost(DataSet: TDataSet);
begin
  ListBox1.Items.Add( 'DataSet.Name: ' + 'Before post' );
end;

procedure TMainform.btnApplyUpdatesClick(Sender: TObject);
begin
  FDDataSet.ApplyUpdates( 0 );

end;

procedure TMainform.btnCopyDataSetClick(Sender: TObject);
var
  CopyDataSetOptions: TFDCopyDataSetOptions;
begin
  ListBox1.Clear;
  if FDDataSet <> nil then
    FreeAndNil( FDDataSet );
  case rgpDestinationType.ItemIndex of
    0: begin
         FDDataSet := TFDQuery.Create( Self );
         TFDQuery(FDDataSet).Connection := SharedDMVcl.FDConnection;
         TFDQuery(FDDataSet).Sql.Text := 'SELECT * FROM ' + destTabName.Text;
         FDDataSet.Open;
         btnApplyUpdates.Enabled := cbxCachedDestUpdates.Checked;
       end;
    1: begin
         FDDataSet := TFDMemTable.Create( Self );
         btnApplyUpdates.Enabled := False;
       end;
  end;
  FDDataSet.CachedUpdates := cbxCachedDestUpdates.Checked;
  WireUpEventHandlers( FDDataSet );
  CopyDataSetOptions := BuildCopyDataSetOptions;
  FDDataSet.CopyDataSet( srcFDQuery, CopyDataSetOptions );
  destinationDataSource.DataSet := FDDataSet;
end;

procedure TMainform.btnCreateDestinationClick(Sender: TObject);
var
  TabList: TStringList;
  TabName: string;
begin
  IsDestTableNameProbably;
  // Drop destination table, if exists
  TabList := TStringList.Create;
  TabName := destTabName.Text;
  try
    SharedDMVcl.FDConnection.GetTableNames( '', '', '', TabList);//, [tkTable, tkTempTable, tkLocalTable],);
    if TabList.IndexOf( destTabName.Text ) > -1 then
      SharedDMVcl.FDConnection.ExecSQL('DROP TABLE ' + destTabName.Text );
  finally
    TabList.Free;
  end;

  SharedDMVcl.FDConnection.ExecSQL( memCreateSQL.Lines.Text );

end;

procedure TMainform.btnCreateSourceClick(Sender: TObject);
begin
  if srcFDQuery <> nil then
    FreeAndNil( srcFDQuery );
  srcFDQuery := TFDQuery.Create( Self );
  srcFDQuery.Connection := SharedDMVcl.FDConnection;
  srcFDQuery.CachedUpdates := cbxCachedSourceUpdates.Checked;
  srcFDQuery.SQL.Text := edtSQLSelect.Text;
  srcFDQuery.Open;
  sourceDataSource.DataSet := srcFDQuery;
end;

function TMainform.BuildCopyDataSetOptions: TFDCopyDataSetOptions;
begin
  Result := [];
  if cbxStructure.Checked then Result := Result + [coStructure];
  if cbxRestart.Checked then Result := Result + [coRestart];
  if cbxAppend.Checked then Result := Result + [coAppend];
  if cbxEdit.Checked then Result := Result + [coEdit];
  {$IF CompilerVersion > 27.0} // Added in Delphi XE7
  if cbxDelete.Checked then Result := Result + [coDelete];
  {$ENDIF}
  if cbxRefresh.Checked then Result := Result + [coRefresh];
  if cbxCalcFields.Checked then Result := Result + [coCalcFields];
  if cbxAggregatesReset.Checked then Result := Result + [coAggregatesReset];
  if cbxAggregatesCopy.Checked then Result := Result + [coAggregatesCopy];
  if cbxConstraintsReset.Checked then Result := Result + [coConstraintsReset];
  if cbxConstraintsCopy.Checked then Result := Result + [coConstraintsCopy];
  if cbxIndexesReset.Checked then Result := Result + [coIndexesReset];
  if cbxIndexesCopy.Checked then Result := Result + [coIndexesCopy];
end;

function TMainform.IsDestTableNameProbably: Boolean;
var
  createSQL: string;
begin
  Result := False;
  createSQL := memCreateSQL.Lines.Text.ToUpper.Trim;
  if createSQL.StartsWith('CREATE TABLE ') then
  begin
    createSQL := Copy( createSQL, 14, MaxInt );
    // Do this again with TABLE
    if createSQL.StartsWith( UpperCase(destTabName.Text).Trim) then
    begin
      createSQL := Copy( createSQL, Length(Trim(destTabName.Text)) + 1, MaxInt );
      if createSQL.TrimLeft[1] in [' ', '(' ] then
        exit( True );
    end;
  end;
  raise Exception.Create('Destination table name in SQL CREATE ' +
                         'statement and Destination TableName field ' +
                         'do not match. Cannot continue');
end;

procedure TMainform.WireUpEventHandlers(FDDataSet: TFDDataSet);
begin
  FDDataSet.BeforeDelete := BeforeDelete;
  FDDataSet.BeforeInsert := BeforeInsert;
  FDDataSet.BeforePost := BeforePost;
  FDDataSet.AfterDelete := AfterDelete;
  FDDataSet.AfterInsert := AfterInsert;
  FDDataSet.AfterPost := AfterPost;
end;

end.
