(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, Vcl.StdCtrls,
  Datasnap.DBClient, Data.DB, FireDAC.Comp.DataSet, FireDAC.Comp.Client,
  Vcl.Grids, Vcl.DBGrids, Vcl.ComCtrls, System.Diagnostics;

type
  TForm1 = class(TForm)
    CustomerQuery: TFDQuery;
    EmployeeQuery: TFDQuery;
    ClientDataSet1: TClientDataSet;
    CopyDataSetGroup: TGroupBox;
    btnUseCopyDataSet: TButton;
    Label1: TLabel;
    lbxDataSet: TListBox;
    FDMemTable1: TFDMemTable;
    cbxStructure: TCheckBox;
    cbxRestart: TCheckBox;
    cbxAppend: TCheckBox;
    cbxEdit: TCheckBox;
    cbxDelete: TCheckBox;
    cbxRefresh: TCheckBox;
    cbxAggregatesReset: TCheckBox;
    cbxAggregatesCopy: TCheckBox;
    cbxConstraintsReset: TCheckBox;
    cbxConstraintsCopy: TCheckBox;
    cbxIndexesReset: TCheckBox;
    cbxIndexesCopy: TCheckBox;
    GroupBox1: TGroupBox;
    Label2: TLabel;
    lbxFDDataSet: TListBox;
    btnUseData: TButton;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    StatusBar1: TStatusBar;
    cbxCalcFields: TCheckBox;
    CustomerQueryCalcField: TStringField;
    procedure FormCreate(Sender: TObject);
    procedure btnUseCopyDataSetClick(Sender: TObject);
    procedure btnUseDataClick(Sender: TObject);
    procedure FDMemTable1AfterInsert(DataSet: TDataSet);
  private
    { Private declarations }
    function BuildCopyDataSetOptions: TFDCopyDataSetOptions;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  StopWatch: TStopWatch;

procedure Start;
procedure Complete;

implementation

{$R *.dfm}

uses SharedDMVclU, DataPaths;

procedure Start;
begin
  StopWatch := TStopWatch.StartNew;
end;

procedure Complete;
begin
  StopWatch.Stop;
  Form1.StatusBar1.Panels[0].Text := 'Elapse time: ' +
    StopWatch.ElapsedMilliseconds.ToString + ' milliseconds';
end;

procedure TForm1.btnUseDataClick(Sender: TObject);
var
  FDDataSet: TFDDataSet;
begin
  if lbxFDDataSet.ItemIndex = -1 then
    ShowMessage('Select a FireDAC dataset whose data to copy')
  else
  begin
    if FDMemTable1.Active then
      FDMemTable1.Close;
    FDDataSet := TFDDataSet(Self.FindComponent( lbxFDDataSet.Items[lbxFDDataSet.ItemIndex]));
    if FDDataSet.Active then FDDataSet.Close;
    FDDataSet.Open;
    Start;
    FDMemTable1.Data := FDDataSet.Data;
    Complete;
  end;
end;

function TForm1.BuildCopyDataSetOptions: TFDCopyDataSetOptions;
begin
  result := [];
  if cbxStructure.Checked then Result := Result + [coStructure];
  if cbxRestart.Checked then Result := Result + [coRestart];
  if cbxAppend.Checked then Result := Result + [coAppend];
  if cbxEdit.Checked then Result := Result + [coEdit];
  {$IF CompilerVersion > 27.0} // Added in Delphi XE7
  if cbxDelete.Checked then Result := Result + [coDelete];
  {$ENDIF}
  if cbxRefresh.Checked then Result := Result + [coRefresh];
  if cbxCalcFields.Checked then Result := Result + [coCalcFields];
  if cbxAggregatesReset.Checked then Result := Result + [coAggregatesReset];
  if cbxAggregatesCopy.Checked then Result := Result + [coAggregatesCopy];
  if cbxConstraintsReset.Checked then Result := Result + [coConstraintsReset];
  if cbxConstraintsCopy.Checked then Result := Result + [coConstraintsCopy];
  if cbxIndexesReset.Checked then Result := Result + [coIndexesReset];
  if cbxIndexesCopy.Checked then Result := Result + [coIndexesCopy];
end;

procedure TForm1.btnUseCopyDataSetClick(Sender: TObject);
var
  CopyDataSetOptions: TFDCopyDataSetOptions;
  DataSet: TDataSet;
begin
  if lbxDataSet.ItemIndex = -1 then
    ShowMessage('Select a dataset whose data to copy')
  else
  begin
    CopyDataSetOptions := BuildCopyDataSetOptions;
    DataSet := TDataSet(Self.FindComponent( lbxDataSet.Items[lbxDataSet.ItemIndex]));
    Start;
    FDMemTable1.CopyDataSet( DataSet, CopyDataSetOptions );
    Complete;
  end;
end;

procedure TForm1.FDMemTable1AfterInsert(DataSet: TDataSet);
begin
  StatusBar1.SimpleText := 'Inserting record ' + FDMemTable1.RecNo.ToString;
  Application.ProcessMessages;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  SharedDMVcl.FDConnection.Open();
  ClientDataSet1.FileName := SamplePath + '\country.xml';
  for i := 0 to Self.ComponentCount - 1 do
    if (Self.Components[i] is TDataSet) and
      not ( Self.Components[i] is TFDMemTable ) then
    begin
      TDataSet(Self.Components[i]).Open;
      lbxDataSet.Items.Add( Self.Components[i].Name );
      if Self.Components[i] is TFDDataSet then
        lbxFDDataSet.Items.Add( Self.Components[i].Name );
    end;
end;

end.
