(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, Data.DB, Vcl.StdCtrls,
  Vcl.Grids, Vcl.DBGrids, FireDAC.Comp.DataSet, FireDAC.Comp.Client;

type
  TForm1 = class(TForm)
    FDQuery1: TFDQuery;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    Label1: TLabel;
    Label2: TLabel;
    btnTransactionGoodQuery: TButton;
    btnTransactionBadQuery: TButton;
    Label3: TLabel;
    btnNoTransaction: TButton;
    FDQuery2: TFDQuery;
    procedure btnTransactionBadQueryClick(Sender: TObject);
    procedure btnNoTransactionBadQueryClick(Sender: TObject);
    procedure btnTransactionGoodQueryClick(Sender: TObject);
  private
    procedure RemoveTestRecordsIfExist;
    { Private declarations }
  public
    { Public declarations }
    procedure UpdateDBGrid;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses SharedDMVclU;

resourcestring
  SQLstring = 'INSERT INTO Customer (Customer, Contact_First, Contact_Last, City, Country) ' +
              '  VALUES ( :cust, :C_First, :C_Last, :City, :Country)';

procedure TForm1.btnTransactionBadQueryClick(Sender: TObject);
begin
  try
    RemoveTestRecordsIfExist;
    FDQuery1.SQL.Text := SQLstring;
    FDQuery1.Prepare;
    try
      SharedDMVcl.FDConnection.StartTransaction;
      try
        FDQuery1.Params[0].Value := 'Absolute Good';
        FDQuery1.Params[1].Value := 'Adrian';
        FDQuery1.Params[2].Value := 'Albright';
        FDQuery1.Params[3].Value := 'Allanville';
        FDQuery1.Params[4].Value := 'USA';
        FDQuery1.ExecSQL;
        FDQuery1.Params[0].Value := 'Bagle Bonanza';
        FDQuery1.Params[1].Value := 'Brian';
        FDQuery1.Params[2].Value := 'Brukker';
        FDQuery1.Params[3].Value := 'Blackpool';
        FDQuery1.Params[4].Value := 'United Kingdom';
        FDQuery1.ExecSQL;
        SharedDMVcl.FDConnection.Commit;
      except
        SharedDMVcl.FDConnection.Rollback;
      end;
    finally
      FDQuery1.Unprepare;
    end;
  finally
    UpdateDBGrid;
  end;
end;

procedure TForm1.btnTransactionGoodQueryClick(Sender: TObject);
begin
  try
    RemoveTestRecordsIfExist;
    FDQuery1.SQL.Text := SQLstring;
    FDQuery1.Prepare;
    try
      SharedDMVcl.FDConnection.StartTransaction;
      try
        FDQuery1.Params[0].Value := 'Absolute Good';
        FDQuery1.Params[1].Value := 'Adrian';
        FDQuery1.Params[2].Value := 'Albright';
        FDQuery1.Params[3].Value := 'Allanville';
        FDQuery1.Params[4].Value := 'USA';
        FDQuery1.ExecSQL;
        FDQuery1.Params[0].Value := 'Bagle Bonanza';
        FDQuery1.Params[1].Value := 'Brian';
        FDQuery1.Params[2].Value := 'Brukker';
        FDQuery1.Params[3].Value := 'Blackpool';
        FDQuery1.Params[4].Value := 'England';
        FDQuery1.ExecSQL;
        SharedDMVcl.FDConnection.Commit;
      except
        SharedDMVcl.FDConnection.Rollback;
      end;
    finally
      FDQuery1.Unprepare;
    end;
  finally
    UpdateDBGrid;
  end;
end;

procedure TForm1.btnNoTransactionBadQueryClick(Sender: TObject);
begin
  try
    RemoveTestRecordsIfExist;
    FDQuery1.SQL.Text := SQLstring;
    FDQuery1.Prepare;
    try
      FDQuery1.Params[0].Value := 'Absolute Good';
      FDQuery1.Params[1].Value := 'Adrian';
      FDQuery1.Params[2].Value := 'Albright';
      FDQuery1.Params[3].Value := 'Allanville';
      FDQuery1.Params[4].Value := 'USA';
      FDQuery1.ExecSQL;
      FDQuery1.Params[0].Value := 'Bagle Bonanza';
      FDQuery1.Params[1].Value := 'Brian';
      FDQuery1.Params[2].Value := 'Brukker';
      FDQuery1.Params[3].Value := 'Blackpool';
      FDQuery1.Params[4].Value := 'United Kingdom';
      FDQuery1.ExecSQL;
    finally
      FDQuery1.Unprepare;
    end;
  finally
    UpdateDBGrid;
  end;
end;

procedure TForm1.RemoveTestRecordsIfExist;
begin
  // Remove the test records, if they exist
  FDQuery1.SQL.Text := 'DELETE FROM Customer WHERE Customer = :n1 ' +
                       ' OR Customer = :n2';
  FDQuery1.Params[0].Value := 'Absolute Good';
  FDQuery1.Params[1].Value := 'Bagle Bonanza';
  FDQuery1.ExecSQL;
end;

procedure TForm1.UpdateDBGrid;
begin
  FDQuery2.Close;
  Application.ProcessMessages;
  FDQuery2.SQL.Text := 'SELECT * FROM Customer';
  FDQuery2.Open;
  FDQuery2.Last;
end;

end.
