(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Windows, Messages, System.SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, ExtCtrls, Menus, ComCtrls, DBGrids, DB,
  DBClient, DBCtrls, FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Param,
  FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf,
  FireDAC.Comp.DataSet, FireDAC.Comp.Client, FireDAC.Stan.StorageXML,
  FireDAC.Stan.StorageBin, System.Diagnostics,
  FireDAC.Stan.ExprFuncs;   // This unit is critical if you want to use FireDAC functions

type
  TForm1 = class(TForm)
    StatusBar1: TStatusBar;
    MainMenu1: TMainMenu;
    Panel1: TPanel;
    Splitter1: TSplitter;
    Panel2: TPanel;
    Label1: TLabel;
    Splitter2: TSplitter;
    Panel3: TPanel;
    DBNavigator1: TDBNavigator;
    Panel4: TPanel;
    StringGrid1: TStringGrid;
    DBGrid1: TDBGrid;
    ClearGridBtn: TButton;
    File1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    VisitwwwDephiDevelperDayscom1: TMenuItem;
    VisitwwwJensenDataSystemscom1: TMenuItem;
    N3: TMenuItem;
    About1: TMenuItem;
    Close1: TMenuItem;
    DataSource1: TDataSource;
    IndexFieldNamesEdit: TEdit;
    SelectIndexBtn: TButton;
    SetRangeBtn: TButton;
    ApplyRangeBtn: TButton;
    Label2: TLabel;
    ApplyFilterExpressionBtn: TButton;
    FirstBtn: TButton;
    PriorBtn: TButton;
    NextBtn: TButton;
    LastBtn: TButton;
    CancelRangeBtn: TButton;
    GroupBox1: TGroupBox;
    CaseInsCheckBox: TCheckBox;
    NoPartialCheckBox: TCheckBox;
    FilterExpressionEdit: TEdit;
    ApplyDropFilterBtn: TButton;
    cbxStartExclusive: TCheckBox;
    FDMemTable1: TFDMemTable;
    FDStanStorageXMLLink1: TFDStanStorageXMLLink;
    FDMemTable1GUID: TStringField;
    FDMemTable1FirstName: TStringField;
    FDMemTable1LastName: TStringField;
    FDMemTable1Address: TStringField;
    FDMemTable1City: TStringField;
    FDMemTable1State: TStringField;
    FDMemTable1ZipCode: TStringField;
    FDMemTable1RandomNumber: TIntegerField;
    cbxEndExclusive: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure VisitwwwDephiDevelperDayscom1Click(Sender: TObject);
    procedure VisitwwwJensenDataSystemscom1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure FDMemTable1AfterOpen(DataSet: TDataSet);
    procedure FDMemTable1AfterClose(DataSet: TDataSet);
    procedure Close1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure ClearGridBtnClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure SelectIndexBtnClick(Sender: TObject);
    procedure IndexFieldNamesEditChange(Sender: TObject);
    procedure SetRangeBtnClick(Sender: TObject);
    procedure ApplyRangeBtnClick(Sender: TObject);
    procedure CancelRangeBtnClick(Sender: TObject);
    procedure CaseInsCheckBoxClick(Sender: TObject);
    procedure NoPartialCheckBoxClick(Sender: TObject);
    procedure ApplyFilterExpressionBtnClick(Sender: TObject);
    procedure FirstBtnClick(Sender: TObject);
    procedure PriorBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure LastBtnClick(Sender: TObject);
    procedure ApplyDropFilterBtnClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure FDMemTable1FilterRecord(DataSet: TDataSet; var Accept: Boolean);
  private
    FStopWatch: TStopWatch;
    procedure Start;
    procedure Complete;
    procedure LoadFDMemTable(FileName: String);
    function GetMaxRangeItems: Integer;
    procedure UpdateGridLabels;
    function GetTemporaryIndexFromUser(CurrentIndex: String): String;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  StopWatch: TStopWatch;

procedure Start;
procedure Complete;


implementation

{$R *.dfm}

uses ShellAPI,   //For ShellExecute
     DataPaths;

procedure Start;
begin
  StopWatch := TStopWatch.StartNew;
end;

procedure Complete;
begin
  StopWatch.Stop;
  Form1.StatusBar1.Panels[0].Text := 'Elapse time: ' +
    StopWatch.ElapsedMilliseconds.ToString + ' milliseconds';
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FDMemTable1.LoadFromFile( DataPaths.BigMemTablePath, sfXml );
  StringGrid1.RowCount := 2;
  StringGrid1.ColCount := 3;
  StringGrid1.Cells[0,0] := 'Field Name';
  StringGrid1.Cells[1,0] := 'Start Range';
  StringGrid1.Cells[2,0] := 'End Range';
  StringGrid1.FixedRows := 1;
  StringGrid1.FixedCols := 1;
end;

procedure TForm1.LoadFDMemTable(FileName: String);
begin
  ClearGridBtnClick(Self);
  if FDMemTable1.Active then
    FDMemTable1.Close;
  FDMemTable1.IndexFieldNames := '';
  FDMemTable1.Filtered := False;
  FDMemTable1.Filter := '';
  FDMemTable1.FileName := FileName;
  FDMemTable1.Open;
  IndexFieldNamesEdit.ReadOnly := False;
  IndexFieldNamesEdit.Text := FDMemTable1.Fields[0].FieldName;
  IndexFieldNamesEdit.ReadOnly := True;
end;

procedure TForm1.UpdateGridLabels;
var
  IndexFields: String;
  SList: TStringList;
  i: Integer;
begin
  IndexFields := IndexFieldNamesEdit.Text;
  SList := TStringList.Create;
  SList.Delimiter := ';';
  SList.DelimitedText := IndexFields;
  try
    case SList.Count of
      0     : begin
                StringGrid1.RowCount := 2;
                StringGrid1.Cells[0, 1] := '';
              end;
      1     : StringGrid1.RowCount := 2;
      2..255: StringGrid1.RowCount := Succ(SList.Count);
    end;
    for i := 0 to Pred(SList.Count) do
      StringGrid1.Cells[0, Succ(i)] := SList[i];
  finally
     SList.Free;
  end;
end;

procedure TForm1.Start;
begin
  FStopWatch := TStopWatch.StartNew;
end;

procedure TForm1.Complete;
begin
  FStopWatch.Stop;
  StatusBar1.Panels[0].Text := 'Elapse time: ' +
    FStopWatch.ElapsedMilliseconds.ToString + ' milliseconds';
end;

procedure TForm1.VisitwwwDephiDevelperDayscom1Click(Sender: TObject);
begin
ShellExecute(Application.Handle, 'open',
 'http://www.DelphiDeveloperDays.com', nil, nil, SW_SHOW);
end;

procedure TForm1.VisitwwwJensenDataSystemscom1Click(Sender: TObject);
begin
ShellExecute(Application.Handle, 'open',
 'http://www.JensenDataSystems.com', nil, nil, SW_SHOW);
end;

procedure TForm1.About1Click(Sender: TObject);
begin
ShowMessage('Filter (c) 2002-2017 Cary Jensen, Jensen Data Systems, Inc. '
  + #10#13
  + #10#13 + 'Demonstration of how to filter an FDMemTable based on its data.'
  + #10#13
  + #10#13 + 'This project is provided for demonstration purposes only.'
  + #10#13
  + #10#13 + 'Visit http://www.JensenDataSystems.com for information about training and consulting services.');
end;

procedure TForm1.FDMemTable1AfterOpen(DataSet: TDataSet);
var
  i: Integer;
begin
  Open1.Enabled := False;
  Close1.Enabled := True;
  for i := 0 to Pred(Self.ComponentCount) do
    if (Self.Components[i] is TButton) and
      (Components[i].Name <> 'CancelRangeBtn') then
      TButton(Components[i]).Enabled := True;
end;

procedure TForm1.FDMemTable1FilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
begin
//  Accept := FDMemTable1.Fields[1].AsString = 'Scarlett';
end;

procedure TForm1.FDMemTable1AfterClose(DataSet: TDataSet);
var
  i: Integer;
begin
  Open1.Enabled := True;
  Close1.Enabled := False;
  for i := 0 to Pred(Self.ComponentCount) do
    if Self.Components[i] is TButton then
      TButton(Components[i]).Enabled := False;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
  FDMemTable1.Close;
  ClearGridBtnClick(Self);
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  FDMemTable1.Open;
  ClearGridBtnClick(Self);
end;

procedure TForm1.ClearGridBtnClick(Sender: TObject);
var
  Col, Row: Integer;
begin
for Col := 1 to 2 do
  for Row := 1 to StringGrid1.RowCount do
    StringGrid1.Cells[Col, Row] := '';
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
Close;
end;

type
  EBadFieldInIndex = class(Exception);

function TForm1.GetTemporaryIndexFromUser(
  CurrentIndex: String): String;
  //local function to verify field list

  function FieldsValid(FDDataSet: TFDDataSet;
    FieldList: String): Boolean;
  var
    i: Integer;
    SList: TStringList;
    s1, s2: string;
  begin
    Result := False;
    SList := TStringlist.Create;
    SList.Delimiter := ';';
    SList.DelimitedText := FieldList;
    try
      if SList.Count = 0 then
      begin
        Exit( True );
      end
      else
        for s1 in SList do
        begin
          if UpperCase(s1).EndsWith(':D') or
             UpperCase(s1).EndsWith(':A') or
             UpperCase(s1).EndsWith(':N') then
            s2 := copy( s1, 1, Length(s1) - 2)
          else
            s2 := s1;
          if FDDataSet.FindField(s2) = nil then
            exit( False );
        end;
        Result := True;
    finally
      SList.Free;
    end;
  end; //function FieldsValid

begin
  Result := IndexFieldNamesEdit.Text;
  if InputQuery('Enter index field(s) [ex: field1;field2:D]',
    'Index on', Result) then
  begin
    while Pos(' ',Result) <> 0  do
      Result := StringReplace(Result, ' ',
        '', [rfReplaceAll]);
    if not FieldsValid(FDMemTable1, Result) then
      raise EBadFieldInIndex.Create('IndexFieldNames ' +
        'contains at least one invalid field name');
    Result := Result;
  end; //if InputQuery
end;


procedure TForm1.SelectIndexBtnClick(Sender: TObject);
var
  NewIndexFields: String;
begin
  NewIndexFields := GetTemporaryIndexFromUser(IndexFieldNamesEdit.Text);
  if NewIndexFields <> IndexFieldNamesEdit.Text then
  begin
    IndexFieldNamesEdit.Text := NewIndexFields;
    ClearGridBtnClick(Sender);
  end;
end;

procedure TForm1.IndexFieldNamesEditChange(Sender: TObject);
begin
  Start;
  FDMemTable1.IndexFieldNames := IndexFieldNamesEdit.Text;
  Complete;
  UpdateGridLabels;
end;

procedure TForm1.SetRangeBtnClick(Sender: TObject);
var
  MaxItems: Integer;
  arLow: array of TVarRec;
  arHigh: array of TVarRec;
  i: Integer;
begin
if FDMemTable1.IndexFieldNames = '' then
  begin
    ShowMessage('Set index field names before trying to set a range');
    Exit;
  end;
  MaxItems := GetMaxRangeItems;
  SetLength( arLow, MaxItems);
  SetLength( arHigh, MaxItems);
  for i := 1 to MaxItems do
  begin
    arLow[i-1].vtype := vtUnicodeString;
    arLow[i-1].VUnicodeString := Pointer((StringGrid1.Cells[ 1, i]));
    arHigh[i-1].vtype := vtUnicodeString;
    arHigh[i-1].VUnicodeString := Pointer((StringGrid1.Cells[ 2, i]));
  end;
  Start;
  FDMemTable1.SetRange( arLow, arHigh,
                        cbxStartExclusive.Checked,
                        cbxEndExclusive.Checked );
  Complete;
  CancelRangeBtn.Enabled := True;

  (* Note: The SetRange method works with n (2+) fields of the index
   * so long as the first n-1 fields define the same start and end
   * value on the range. For example, the following will work:
   * CDS.SetRange([1, 1], [1, 20];
   * but the following will not work:
   *  CDS.SetRange([1, 1], [2, 20];
   *)
//  case MaxItems of
//  0: ShowMessage('Enter a range');
//  1: begin
//     Start;
//     FDMemTable1.SetRange(
//       [StringGrid1.Cells[1,1]],
//       [StringGrid1.Cells[2,1]]);
//     Complete;
//     end;
//  2: begin
//     Start;
//     FDMemTable1.SetRange(
//       [StringGrid1.Cells[1,1],
//       StringGrid1.Cells[1,2]],
//       [StringGrid1.Cells[2,1],
//       StringGrid1.Cells[2,2]]);
//     Complete;
//     end;
//  3:  begin
//      Start;
//      FDMemTable1.SetRange(
//        [StringGrid1.Cells[1,1],
//        StringGrid1.Cells[1,2],
//        StringGrid1.Cells[1,3]],
//        [StringGrid1.Cells[2,1],
//        StringGrid1.Cells[2,2],
//        StringGrid1.Cells[2,3]]);
//      Complete;
//      end;
//  4:  begin
//      Start;
//      FDMemTable1.SetRange(
//        [StringGrid1.Cells[1,1],
//        StringGrid1.Cells[1,2],
//        StringGrid1.Cells[1,3],
//        StringGrid1.Cells[1,4]],
//        [StringGrid1.Cells[2,1],
//        StringGrid1.Cells[2,2],
//        StringGrid1.Cells[2,3],
//        StringGrid1.Cells[2,4]]);
//      Complete;
//      end;
//  5:  begin
//      Start;
//      FDMemTable1.SetRange(
//        [StringGrid1.Cells[1,1],
//        StringGrid1.Cells[1,2],
//        StringGrid1.Cells[1,3],
//        StringGrid1.Cells[1,4],
//        StringGrid1.Cells[1,5]],
//        [StringGrid1.Cells[2,1],
//        StringGrid1.Cells[2,2],
//        StringGrid1.Cells[2,3],
//        StringGrid1.Cells[2,4],
//        StringGrid1.Cells[2,5]]);
//      Complete;
//      end;
//  6..MaxInt: ShowMessage('This example project limited to '+
//    'a maximum of 5 fields in a range');
//  end;
//  if MaxItems < 6 then
//    CancelRangeBtn.Enabled := True;
end;

procedure TForm1.ApplyRangeBtnClick(Sender: TObject);
var
  MaxItems: Integer;
  i: Integer;
begin
if FDMemTable1.IndexFieldNames = '' then
  begin
    ShowMessage('Set index field names before trying to set a range');
    Exit;
  end;
  MaxItems := GetMaxRangeItems;
  if MaxItems = 0 then
  begin
    ShowMessage('Enter a range');
    Exit;
  end;
  Start;
  FDMemTable1.SetRangeStart;
  for i := 1 to MaxItems do
    FDMemTable1.FieldByName(StringGrid1.Cells[0,i]).Value :=
      Trim(StringGrid1.Cells[1,i]);
  FDMemTable1.SetRangeEnd;
  for i := 1 to MaxItems do
    FDMemTable1.FieldByName(StringGrid1.Cells[0,i]).Value :=
      Trim(StringGrid1.Cells[2,i]);
  FDMemTable1.ApplyRange;
  Complete;
  if MaxItems < 6 then
    CancelRangeBtn.Enabled := True;
end;

procedure TForm1.CancelRangeBtnClick(Sender: TObject);
begin
  FDMemTable1.CancelRange;
  CancelRangeBtn.Enabled := False;
end;

procedure TForm1.CaseInsCheckBoxClick(Sender: TObject);
begin
if CaseInsCheckBox.Checked then
  FDMemTable1.FilterOptions :=
    FDMemTable1.FilterOptions + [foCaseInsensitive]
else
  FDMemTable1.FilterOptions :=
    FDMemTable1.FilterOptions - [foCaseInsensitive];
end;

procedure TForm1.NoPartialCheckBoxClick(Sender: TObject);
begin
if NoPartialCheckBox.Checked then
  FDMemTable1.FilterOptions :=
    FDMemTable1.FilterOptions + [foNoPartialCompare]
else
  FDMemTable1.FilterOptions :=
    FDMemTable1.FilterOptions - [foNoPartialCompare];
end;

procedure TForm1.ApplyFilterExpressionBtnClick(Sender: TObject);
begin
  Start;
  FDMemTable1.Filter := FilterExpressionEdit.Text;
  Complete;
end;

procedure TForm1.FirstBtnClick(Sender: TObject);
begin
  Start;
  if not FDMemTable1.FindFirst then
  begin
    Complete;
    ShowMessage('There are no matching records');
  end
  else
    Complete;
end;

procedure TForm1.PriorBtnClick(Sender: TObject);
begin
  Start;
  if not FDMemTable1.FindPrior then
  begin
    Complete;
    ShowMessage('No prior record found');
  end
  else
    Complete;
end;

procedure TForm1.NextBtnClick(Sender: TObject);
begin
  Start;
  FDMemTable1.FindNext;
  Complete;
  if not FDMemTable1.Found then
    ShowMessage('No next record found');
end;

procedure TForm1.LastBtnClick(Sender: TObject);
begin
  Start;
  if not FDMemTable1.FindLast then
  begin
    Complete;
    ShowMessage('There are no matching records');
  end
  else
    Complete;
end;

procedure TForm1.ApplyDropFilterBtnClick(Sender: TObject);
begin
  Start;
  FDMemTable1.Filtered := not FDMemTable1.Filtered;
  Complete;
  if FDMemTable1.Filtered then
    ApplyDropFilterBtn.Caption := 'Drop Filter'
  else
    ApplyDropFilterBtn.Caption := 'Apply Filter';
end;

function TForm1.GetMaxRangeItems: Integer;
var
  i: Integer;
begin
i := 0;
while True do
begin
  if StringGrid1.Cells[1, Succ(i)] <> '' then
    inc(i)
  else
    Break;
end;
Result := i;
i := 0;
while True do
begin
  if StringGrid1.Cells[2, Succ(i)] <> '' then
    inc(i)
  else
    Break;
end;
if Result < i then
  Result := i;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
FDMemTable1.Filter := 'City = '+ QuotedStr('New*') +
   'and [Last Name] = ' + QuotedStr('M*');
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
StatusBar1.Panels[1].Text := IntToStr(FDMemTable1.RecNo) +
  ' of ' + IntToStr(FDMemTable1.RecordCount) + ' records';
end;

end.
