(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, Data.DB, Vcl.StdCtrls,
  Vcl.Grids, Vcl.DBGrids, Vcl.ExtCtrls, Vcl.DBCtrls, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, FireDAC.Stan.StorageBin, FireDAC.Stan.StorageXML,
  FireDAC.Stan.ExprFuncs, Vcl.ComCtrls, System.Diagnostics;

type
  TForm1 = class(TForm)
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    Label1: TLabel;
    cbxIndexes: TComboBox;
    DataSource1: TDataSource;
    FDMemTable1: TFDMemTable;
    FDStanStorageXMLLink1: TFDStanStorageXMLLink;
    StatusBar1: TStatusBar;
    btnRefreshIndexes: TButton;
    procedure FormCreate(Sender: TObject);
    procedure cbxIndexesChange(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure DBGrid1TitleClick(Column: TColumn);
    procedure btnRefreshIndexesClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ListIndexes;
    procedure CreateRuntimeIndexes;
  end;

var
  Form1: TForm1;
  StopWatch: TStopWatch;

procedure Start;
procedure Complete;


function SortFDDataSetWithIndex(DataSet: TFDDataSet;
  const FieldName: String): Boolean;

function SortFDDataSetWithFieldName(DataSet: TFDDataSet;
  const FieldName: String): Boolean;

implementation

{$R *.dfm}

uses SharedDMVclU, DataPaths, DateUtils, math;

procedure TForm1.btnRefreshIndexesClick(Sender: TObject);
begin
  cbxIndexes.Items.Clear;
  ListIndexes;
end;

procedure TForm1.cbxIndexesChange(Sender: TObject);
var
  s: string;
  Index: TFDIndex;
begin
  if cbxIndexes.ItemIndex <> -1 then
  begin
    if cbxIndexes.ItemIndex = 0 then
    begin
      Start;
      FDMemTable1.IndexName := '';
      Complete;
      exit;
    end
    else
    if cbxIndexes.Text = 'Create a runtime filter-based index' then
    begin
      s := '';
      if InputQuery('Enter pattern for FirstName', 'Pattern', s) then
      begin
        Index := FDMemTable1.Indexes.Add;
        Index.Name := 'FirstName=' + s + 'RTIdx';
        Index.Fields := 'FirstName';
        Index.Filter := '[FirstName] = ''' + s + '''';
        Index.FilterOptions := [ekPartial];
        Index.Active := True;
        cbxIndexes.Items.Add( Index.Name );
        cbxIndexes.ItemIndex := cbxIndexes.Items.Count - 1;
      end
      else
        exit;
    end;
    Start;
    FDMemTable1.IndexName := cbxIndexes.Text;
    Complete;
  end;
end;


procedure TForm1.CreateRuntimeIndexes;
var
  Index: TFDIndex;
begin
  // A field-based index
  Index := FDMemTable1.Indexes.Add;
  Index.Name := 'FirstGuidRTIdx';
  Index.Fields := 'FirstName;GUID:D';
  Index.Active := True;
  // An expression index
  Index := FDMemTable1.Indexes.Add;
  Index.Name := 'RandomRTIdx';
  Index.Expression := 'rand( ' + RandomRange(0, MaxInt).ToString + ')';
  Index.Active := True;
  // A distinct index
  Index := FDMemTable1.Indexes.Add;
  Index.Name := 'CityStateDistinctRTIdx';
  Index.Fields := 'City;State';
  Index.Distinct := True;
  Index.Active := True;
  // A filter index
  Index := FDMemTable1.Indexes.Add;
  Index.Name := 'RubyRTIdx';
  Index.Fields := 'FirstName';
  Index.Filter := '[FirstName] = ''Ruby''';
  Index.Active := True;
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  StatusBar1.Panels[1].Text := IntToStr(FDMemTable1.RecNo) +
    ' of ' + IntToStr(FDMemTable1.RecordCount) + ' records';
end;

procedure TForm1.DBGrid1TitleClick(Column: TColumn);
begin
  Start;
//  SortFDDataSetWithIndex( TFDDataSet(Column.Field.DataSet),
//                          Column.FieldName );
  SortFDDataSetWithFieldName( TFDDataSet(Column.Field.DataSet),
                              Column.FieldName  );
  Complete;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FDMemTable1.IndexDefs.Clear;
  FDMemTable1.LoadFromFile( DataPaths.BigMemTablePath, sfXml );
  CreateRuntimeIndexes;
  ListIndexes;
end;

procedure TForm1.ListIndexes;
var
  i: Integer;
begin
  cbxIndexes.Items.Add('No index');
  for i := 0 to FDMemTable1.Indexes.Count - 1 do
    if FDMemTable1.Indexes[i].Active then
      cbxIndexes.Items.Add( FDMemTable1.Indexes.Items[i].Name );
  cbxIndexes.Items.Add('Create a runtime filter-based index');
  cbxIndexes.ItemIndex := -1;
end;

procedure Start;
begin
  StopWatch := TStopWatch.StartNew;
end;

procedure Complete;
begin
  StopWatch.Stop;
  Form1.StatusBar1.Panels[0].Text := 'Elapse time: ' +
    StopWatch.ElapsedMilliseconds.ToString + ' milliseconds';
end;

function SortFDDataSetWithIndex(DataSet: TFDDataSet;
  const FieldName: String): Boolean;
var
  Field: TField;
  NewIndex: TFDIndex;
type
  iDirection = (idAscending, idDescending);

  function GetIndexName(const FieldName: string; Direction: iDirection): string;
  begin
    if Direction = idAscending then
      Result := FieldName + '_AIdx'
    else
      Result := FieldName + '_DIdx';
  end;

  function CreateIndex(const FieldName: string; Direction: iDirection): TFDIndex;
  begin
    Result := DataSet.Indexes.Add;
    Result.Fields := FieldName;
    Result.Name := GetIndexName(FieldName, Direction);
    if Direction = idDescending then
      Result.DescFields := FieldName;
    Result.Active := True;
  end;

  function FindOrCreateIndex(const FieldName: string; Direction: iDirection): TFDIndex;
  begin
    Result := DataSet.Indexes.FindIndex(GetIndexName(FieldName, Direction));
    if not Assigned(Result) then
      Result := CreateIndex(FieldName, Direction);
    Result.Active := True;
  end;

begin
  Result := False;
  Field := DataSet.Fields.FindField(FieldName);
  if not Assigned(Field) then
    Exit;

  // if field type is not sortable, exit.
  if (Field is TObjectField) or (Field is TBlobField)
    or (Field is TVariantField) or (Field is TBinaryField)
    or not (Field.FieldKind in [ fkData, fkInternalCalc ]) then
    exit;

  if SameText(DataSet.IndexName, GetIndexName(FieldName, idAscending)) then
    NewIndex := FindOrCreateIndex(FieldName, idDescending)
  else
    NewIndex := FindOrCreateIndex(FieldName, idAscending);

  DataSet.IndexName := NewIndex.Name;
  Result := True;
end;


function SortFDDataSetWithFieldName(DataSet: TFDDataSet;
  const FieldName: String): Boolean;
var
  Field: TField;
begin
  Result := False;
  Field := DataSet.Fields.FindField(FieldName);
  if not Assigned(Field) then
    Exit;

  // if field type is not sortable, exit.
  if (Field is TObjectField) or (Field is TBlobField)
    or (Field is TVariantField) or (Field is TBinaryField)
    or not (Field.FieldKind in [ fkData, fkInternalCalc ]) then
    exit;

  if SameText(DataSet.IndexFieldNames, FieldName) or
    SameText(DataSet.IndexFieldNames, FieldName + ':A') then
    DataSet.IndexFieldNames := FieldName + ':D'
  else
    DataSet.IndexFieldNames := FieldName + ':A';

  Result := True;
end;

end.
