(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, Vcl.Grids, Vcl.DBGrids,
  Vcl.ExtCtrls, Vcl.DBCtrls, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Error, FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def,
  FireDAC.Stan.Pool, FireDAC.Stan.Async, FireDAC.Phys, FireDAC.VCLUI.Wait,
  FireDAC.Stan.Param, FireDAC.DatS, FireDAC.DApt.Intf, FireDAC.DApt,
  Datasnap.DBClient, FireDAC.Comp.DataSet, FireDAC.Comp.Client,
  FireDAC.Phys.SQLiteVDataSet, FireDAC.Phys.SQLite,
  FireDAC.Stan.ExprFuncs, Vcl.StdCtrls, FireDAC.Phys.SQLiteDef;

type
  TForm1 = class(TForm)
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    LocalQueryDataSource: TDataSource;
    SQLiteConnection: TFDConnection;
    SalesQuery: TFDQuery;
    EmployeeCDS: TClientDataSet;
    CustomerQuery: TFDQuery;
    FDLocalSQL1: TFDLocalSQL;
    LocalQuery: TFDQuery;
    Label1: TLabel;
    edtTotalOver: TEdit;
    Label2: TLabel;
    cbxCustomerList: TComboBox;
    btnExecQuery: TButton;
    cbxUseOnGetDataSet: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure btnExecQueryClick(Sender: TObject);
    procedure FDLocalSQL1GetDataSet(ASender: TObject; const ASchemaName,
      AName: string; var ADataSet: TDataSet; var AOwned: Boolean);
    procedure cbxUseOnGetDataSetClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses DataPaths, SharedDMVclU;

procedure TForm1.btnExecQueryClick(Sender: TObject);
begin
  LocalQuery.Close;
  if cbxCustomerList.ItemIndex = 0 then
    LocalQuery.MacroByName( 'MoreWhere' ).AsRaw := ''
  else
    LocalQuery.MacroByName( 'MoreWhere' ).AsRaw := 'AND c.Customer = ''' +
                                   cbxCustomerList.Text + ''';';
  LocalQuery.ParamByName( 'v' ).AsString := edtTotalOver.Text;
  LocalQuery.Open();
end;

procedure TForm1.cbxUseOnGetDataSetClick(Sender: TObject);
begin
  if cbxUseOnGetDataSet.Checked then
  begin
    FDLocalSQL1.DataSets.Clear;
    FDLocalSQL1.OnGetDataSet := FDLocalSQL1GetDataSet;
  end
  else
  begin
    FDLocalSQL1.OnGetDataSet := nil;
    FDLocalSQL1.DataSets.Add.DataSet := SalesQuery;
    FDLocalSQL1.DataSets.Add.DataSet := CustomerQuery;
    FDLocalSQL1.DataSets.Add.DataSet := EmployeeCDS;
  end;
end;

procedure TForm1.FDLocalSQL1GetDataSet(ASender: TObject; const ASchemaName,
  AName: string; var ADataSet: TDataSet; var AOwned: Boolean);
begin
///  If run this project in the debugger, and select to use
///  this OnGetDataSet event handler, it will raise an exception.
///  This is normal. This exception will not be seen by the user
///  when running without the debugger.
  AOwned := False;
  if  AName = 'SalesQuery' then
    ADataSet := TDataSet(SalesQuery)
  else if AName = 'CustomerQuery' then
    ADataSet := CustomerQuery
  else if AName = 'EmployeeCDS' then
    ADataSet := EmployeeCDS;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   SalesQuery.Open();
   CustomerQuery.Open();
   EmployeeCDS.LoadFromFile( DataPaths.SamplePath + '\employee.xml' );
   cbxCustomerList.Clear;
   cbxCustomerList.Items.Add('All');
   while not CustomerQuery.Eof do
   begin
     cbxCustomerList.Items.Add(
       CustomerQuery.FieldByName( 'CUSTOMER' ).AsString );
     CustomerQuery.Next;
   end;
   cbxCustomerList.ItemIndex := 0;
   SQLiteConnection.Connected := True;
   FDLocalSQL1.Active := True;
   // This query is parameterized, and uses macro substitution
   LocalQuery.SQL.Text := 'SELECT s.PO_NUMBER, c.Customer, '+
                          '    s.ORDER_DATE, s.TOTAL_VALUE, '+
                          '    e.FirstName || '' '' || e.LastName as "Sales Rep" '+
                          '  FROM SalesQuery s '+
                          '  INNER JOIN CustomerQuery c ON c.CUST_NO = s.CUST_NO '+
                          '  INNER JOIN EmployeeCDS e ON e.EmpNo = s.SALES_REP '+
                          '  WHERE s.TOTAL_VALUE >= :v &MoreWhere ';

   cbxUseOnGetDataSetClick( nil );
end;

end.
