(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Provider, DB, DBClient, ExtCtrls, Grids,
  DBGrids, DBCtrls, Menus,
  FireDAC.Comp.Client,
  FireDAC.Comp.DataSet,
  FireDAC.Comp.UI,
  FireDAC.DApt,
  FireDAC.DApt.Intf,
  FireDAC.DatS,
  FireDAC.Phys,
  FireDAC.Phys.IB,
  FireDAC.Phys.IBBase,
  FireDAC.Phys.Intf,
  FireDAC.Stan.Async,
  FireDAC.Stan.Def,
  FireDAC.Stan.Error,
  FireDAC.Stan.Intf,
  FireDAC.Stan.Option,
  FireDAC.Stan.Param,
  FireDAC.Stan.Pool,
  FireDAC.UI.Intf,
  {$IF CompilerVersion < 28}
  // nothing needed
  {$ELSE}
  FireDAC.Phys.IBDef,
  {$ENDIF}
  FireDAC.VCLUI.Wait;

type
  TForm1 = class(TForm)
    DataSource1: TDataSource;
    Panel1: TPanel;
    DBNavigator2: TDBNavigator;
    DBGrid2: TDBGrid;
    DataSource2: TDataSource;
    Panel2: TPanel;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    Panel3: TPanel;
    Splitter1: TSplitter;
    IncludeCurrentOrderCbx: TCheckBox;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    OrdersByPartNoMemTable: TFDMemTable;
    FDPhysIBDriverLink1: TFDPhysIBDriverLink;
    PartsQuery: TFDQuery;
    PartsSource: TDataSource;
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure FormCreate(Sender: TObject);
    procedure IncludeCurrentOrderCbxClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses ShellApi, SharedDMVclU, DataPaths;

var
  PartFld: TField;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  if not IncludeCurrentOrderCbx.Checked then
    OrdersByPartNoMemTable.Filter := 'OrderNo <> ' +
      QuotedStr(PartsQuery.FieldByName('OrderNo').AsString)
  else
    OrdersByPartNoMemTable.Filter := '';
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  DBIndex: Integer;
begin
  //Set the connection to use the dbdemo.gdb database
  DataPaths.ValidatePath( IBDemoPath );
  // Switch database for
  DBIndex := SharedDMVcl.FDConnection.Params.IndexOfName('Database');
  if DBIndex > -1 then
    SharedDMVcl.FDConnection.Params[DBIndex] :=
      'Database=localhost:' +  DataPaths.IBDemoPath;
  // Get the Items table records
  PartsQuery.Open;
  //Clone the detail cursor.
  OrdersByPartNoMemTable.CloneCursor(PartsQuery, False);
  //Setup the master-detail relationship
  OrdersByPartNoMemTable.MasterSource := PartsSource;
  OrdersByPartNoMemTable.MasterFields := 'PartNo';
  OrdersByPartNoMemTable.IndexFieldNames  := 'PartNo';
  OrdersByPartNoMemTable.IndexesActive := True;
  //The PartsSource.OnDataChange event needs Filtered to be True
  OrdersByPartNoMemTable.Filtered := True;
end;

procedure TForm1.IncludeCurrentOrderCbxClick(Sender: TObject);
begin
  DataSource1DataChange(Self, PartsQuery.Fields[0]);
end;

end.
