(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, FireDAC.UI.Intf,
  FireDAC.VCLUI.Wait, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, FireDAC.Comp.UI, Vcl.Menus, Vcl.Grids, Vcl.DBGrids,
  Vcl.DBCtrls, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls, System.Diagnostics;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Panel2: TPanel;
    FirstBtn: TButton;
    NextBtn: TButton;
    PriorBtn: TButton;
    LastBtn: TButton;
    Panel3: TPanel;
    ScanForwardBtn: TButton;
    ScanBackwardBtn: TButton;
    ControlsStateBtnGrp: TRadioGroup;
    Panel4: TPanel;
    UpDown1: TUpDown;
    Edit1: TEdit;
    MoveByBtn: TButton;
    Panel5: TPanel;
    Edit2: TEdit;
    UpDown2: TUpDown;
    RecNoBtn: TButton;
    StatusBar1: TStatusBar;
    Panel6: TPanel;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Close1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDQuery1: TFDQuery;
    Splitter3: TSplitter;
    Splitter4: TSplitter;
    Panel7: TPanel;
    btnGetBookmark: TButton;
    btnGotoBookmark: TButton;
    procedure FDQuery1AfterClose(DataSet: TDataSet);
    procedure FDQuery1AfterOpen(DataSet: TDataSet);
    procedure Close1Click(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure DataSource1StateChange(Sender: TObject);
    procedure FirstBtnClick(Sender: TObject);
    procedure LastBtnClick(Sender: TObject);
    procedure MoveByBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure PriorBtnClick(Sender: TObject);
    procedure RecNoBtnClick(Sender: TObject);
    procedure ScanBackwardBtnClick(Sender: TObject);
    procedure ScanForwardBtnClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnGetBookmarkClick(Sender: TObject);
    procedure btnGotoBookmarkClick(Sender: TObject);
  private
    { Private declarations }
    FBookmark: TBookmark;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  StopWatch: TStopWatch;

implementation

{$R *.dfm}

uses SharedDMVclU, TypInfo;

procedure Start;
begin
  StopWatch := TStopWatch.StartNew;
end;

procedure Complete;
begin
  StopWatch.Stop;
  Form1.StatusBar1.Panels[0].Text := 'Elapse time: ' +
    StopWatch.ElapsedMilliseconds.ToString + ' milliseconds';
end;

{ TForm1 }

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  StatusBar1.Panels[1].Text := 'Record ' +
    IntToStr(FDQuery1.RecNo) + ' of ' +
    IntToStr(FDQuery1.RecordCount);
  StatusBar1.Panels[3].Text :=
    'BOF = ' + BoolToStr(FDQuery1.Bof, True) + '. ' +
    'EOF = ' + BoolToStr(FDQuery1.Eof, True) + '. ';
end;

procedure TForm1.DataSource1StateChange(Sender: TObject);
begin
  StatusBar1.Panels[2].Text :=
    'State = ' + GetEnumName(TypeInfo(TDataSetState),
    Ord(FDQuery1.State));
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FirstBtnClick(Sender: TObject);
begin
  Start;
  FDQuery1.First;
  Complete;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FDQuery1.Open();
end;

procedure TForm1.NextBtnClick(Sender: TObject);
begin
  Start;
  FDQuery1.Next;
  Complete;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  Start;
  FDQuery1.Open;
  Complete;
end;

procedure TForm1.PriorBtnClick(Sender: TObject);
begin
  Start;
  FDQuery1.Prior;
  Complete;
end;

procedure TForm1.LastBtnClick(Sender: TObject);
begin
  Start;
  FDQuery1.Last;
  Complete;
end;

procedure TForm1.ScanForwardBtnClick(Sender: TObject);
begin
  Start;
  if ControlsStateBtnGrp.ItemIndex = 1 then
    FDQuery1.DisableControls;
    try
      FDQuery1.First;
      while not FDQuery1.Eof do
      begin
        //do something with a record
        FDQuery1.Next;
      end;
    finally
      if ControlsStateBtnGrp.ItemIndex = 1 then
        FDQuery1.EnableControls;
    end;
  Complete;
end;

procedure TForm1.ScanBackwardBtnClick(Sender: TObject);
begin
  Start;
  if ControlsStateBtnGrp.ItemIndex = 1 then
    FDQuery1.DisableControls;
    try
      FDQuery1.Last;
      while not FDQuery1.Bof do
      begin
        //do something with a record
        FDQuery1.Prior;
      end;
    finally
      if ControlsStateBtnGrp.ItemIndex = 1 then
        FDQuery1.EnableControls;
    end;
  Complete;
end;

procedure TForm1.MoveByBtnClick(Sender: TObject);
begin
  Start;
  FDQuery1.MoveBy(UpDown1.Position);
  Complete;
end;

procedure TForm1.RecNoBtnClick(Sender: TObject);
begin
  Start;
  FDQuery1.RecNo := UpDown2.Position;
  Complete;
end;

procedure TForm1.FDQuery1AfterClose(DataSet: TDataSet);
begin
  Open1.Enabled := True;
  Close1.Enabled := False;
end;

procedure TForm1.FDQuery1AfterOpen(DataSet: TDataSet);
begin
  Open1.Enabled := False;
  Close1.Enabled := True;
end;

procedure TForm1.btnGetBookmarkClick(Sender: TObject);
begin
  FBookmark := FDQuery1.GetBookmark;
  btnGotoBookmark.Enabled := True;
end;

procedure TForm1.btnGotoBookmarkClick(Sender: TObject);
begin
  Start;
  FDQuery1.GotoBookmark( FBookmark );
  Complete;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
  FDQuery1.Close;
end;


end.
