(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, FireDAC.Stan.Intf,
  FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS,
  FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.DBCtrls, Vcl.Grids,
  Vcl.DBGrids, FireDAC.Stan.StorageXML;

type
  TForm1 = class(TForm)
    DBGrid1: TDBGrid;
    DBGrid2: TDBGrid;
    DBGrid3: TDBGrid;
    DBNavigator1: TDBNavigator;
    DBNavigator2: TDBNavigator;
    DBNavigator3: TDBNavigator;
    DataSource1: TDataSource;
    DataSource2: TDataSource;
    DataSource3: TDataSource;
    btnCreate: TButton;
    btnSave: TButton;
    btnLoad: TButton;
    btnClear: TButton;
    FDStanStorageXMLLink1: TFDStanStorageXMLLink;
    procedure btnCreateClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
  private
    TopLevelMemTable: TFDMemTable;
    MidLevelMemTable: TFDMemTable;
    ThirdLevelMemTable: TFDMemTable;
    Dir: string;
    procedure CreateNestedDataSets;
    { Private declarations }
  public
    { Public declarations }
  end;
var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.btnClearClick(Sender: TObject);
begin
  TopLevelMemTable.Free;
  btnSave.Enabled := False;
  btnCreate.Enabled := True;
  btnClear.Enabled := False;
end;

procedure TForm1.btnCreateClick(Sender: TObject);
begin
  CreateNestedDataSets;
  btnSave.Enabled := True;
  btnClear.Enabled := True;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
begin
  TopLevelMemTable.SaveToFile(Dir + 'NestedFDMemTable.xml', sfXML);
end;

procedure TForm1.btnLoadClick(Sender: TObject);
begin
  if not FileExists( Dir + 'NestedFDMemTable.xml' ) then
    raise Exception.Create('Save before attempting to load');
  TopLevelMemTable := TFDMemTable.Create(Self);
  MidLevelMemTable := TFDMemTable.Create(TopLevelMemTable);
  ThirdLevelMemTable := TFDMemTable.Create(TopLevelMemTable);
  TopLevelMemTable.LoadFromFile(Dir + 'NestedFDMemTable.xml', sfXML);
  MidLevelMemTable.DataSetField :=
    TDataSetField(TopLevelMemTable.FieldByName('TopNested'));
  ThirdLevelMemTable.DataSetField :=
    TDataSetField(MidLevelMemTable.FieldByName('MidNested'));
  //Configure the DataSources
  DataSource1.DataSet := TopLevelMemTable;
  DataSource2.DataSet := MidLevelMemTable;
  Datasource3.DataSet := ThirdLevelMemTable;

end;

procedure TForm1.CreateNestedDataSets;
begin
  TopLevelMemTable := TFDMemTable.Create(Self);
  MidLevelMemTable := TFDMemTable.Create(TopLevelMemTable);
  ThirdLevelMemTable := TFDMemTable.Create(TopLevelMemTable);

  with TopLevelMemTable.FieldDefs do
  begin
    Add('TopID', ftInteger);
    Add('TopName', ftString, 40);
    Add('TopComments', ftMemo);
    Add('TopDateInitiated', ftDate);
  end;
  with TopLevelMemTable.FieldDefs.AddFieldDef do
  begin
    Name := 'TopNested';
    DataType := ftDataSet;
    with ChildDefs do
    begin
      Add('MidID', ftInteger);
      Add('MidName', ftString, 30);
      with AddChild do
      begin
        Name := 'MidNested';
        DataType := ftDataSet;
        with ChildDefs do
        begin
          Add('ThirdID', ftInteger);
          Add('ThirdName', ftString, 25);
          Add('ThirdActive', ftBoolean);
        end;
      end;
    end;
  end;
  //Create the FDMemTable and its nested datasets
  TopLevelMemTable.Active := True;
  //Hook up the other FDMemTables
  MidLevelMemTable.DataSetField :=
    TDataSetField(TopLevelMemTable.FieldByName('TopNested'));
  ThirdLevelMemTable.DataSetField :=
    TDataSetField(MidLevelMemTable.FieldByName('MidNested'));
  //Configure the DataSources
  DataSource1.DataSet := TopLevelMemTable;
  DataSource2.DataSet := MidLevelMemTable;
  Datasource3.DataSet := ThirdLevelMemTable;
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  dir := ExtractFileDir( ParamStr(0) );
end;

end.
