(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB, FireDAC.Stan.Intf,
  FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS,
  FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.DBCtrls, Vcl.Grids,
  Vcl.DBGrids, FireDAC.Stan.StorageXML;

type
  TForm1 = class(TForm)
    DBGrid1: TDBGrid;
    DBGrid2: TDBGrid;
    DBGrid3: TDBGrid;
    DBNavigator1: TDBNavigator;
    DBNavigator2: TDBNavigator;
    DBNavigator3: TDBNavigator;
    DataSource1: TDataSource;
    DataSource2: TDataSource;
    DataSource3: TDataSource;
    btnCreate: TButton;
    btnSave: TButton;
    btnLoad: TButton;
    btnClear: TButton;
    FDStanStorageXMLLink1: TFDStanStorageXMLLink;
    procedure btnCreateClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
  private
    TopLevelMemTable: TFDMemTable;
    MidLevelMemTable: TFDMemTable;
    ThirdLevelMemTable: TFDMemTable;
    Dir: string;
    procedure CreateNestedDataSets;
    { Private declarations }
  public
    { Public declarations }
  end;
var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.btnClearClick(Sender: TObject);
begin
  TopLevelMemTable.Free;
  btnSave.Enabled := False;
  btnCreate.Enabled := True;
  btnClear.Enabled := False;
end;

procedure TForm1.btnCreateClick(Sender: TObject);
begin
  CreateNestedDataSets;
  btnSave.Enabled := True;
  btnClear.Enabled := True;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
begin
  TopLevelMemTable.SaveToFile(Dir + 'NestedFDMemTable.xml', sfXML);
end;

procedure TForm1.btnLoadClick(Sender: TObject);
begin
  if not FileExists( Dir + 'NestedFDMemTable.xml' ) then
    raise Exception.Create('Save before attempting to load');
  TopLevelMemTable := TFDMemTable.Create(Self);
  MidLevelMemTable := TFDMemTable.Create(TopLevelMemTable);
  ThirdLevelMemTable := TFDMemTable.Create(TopLevelMemTable);
  TopLevelMemTable.LoadFromFile(Dir + 'NestedFDMemTable.xml', sfXML);
  MidLevelMemTable.DataSetField :=
    TDataSetField(TopLevelMemTable.FieldByName('TopNested'));
  ThirdLevelMemTable.DataSetField :=
    TDataSetField(MidLevelMemTable.FieldByName('MidNested'));
  //Configure the DataSources
  DataSource1.DataSet := TopLevelMemTable;
  DataSource2.DataSet := MidLevelMemTable;
  Datasource3.DataSet := ThirdLevelMemTable;

end;

procedure TForm1.CreateNestedDataSets;
begin
  TopLevelMemTable := TFDMemTable.Create(Self);
  MidLevelMemTable := TFDMemTable.Create(TopLevelMemTable);
  ThirdLevelMemTable := TFDMemTable.Create(TopLevelMemTable);

  with TIntegerField.Create(Self) do
  begin
    Name := 'TopID';
    FieldKind := fkData;
    FieldName := 'ID';
    DataSet := TopLevelMemTable;
    Required := True;
  end;
  with TStringField.Create(Self) do
  begin
    Name := 'TopName';
    FieldKind := fkData;
    FieldName := 'Name';
    Size := 40;
    DataSet := TopLevelMemTable;
  end;
  with TMemoField.Create(Self) do
  begin
    Name := 'TopComments';
    FieldKind := fkData;
    FieldName := 'Comments';
    DataSet := TopLevelMemTable;
  end;
  with TDateField.Create(Self) do
  begin
    Name := 'TopDateInitiated';
    FieldKind := fkData;
    FieldName := 'Date Initiated';
    DataSet := TopLevelMemTable;
  end;
  //Note: For TDataSetFields, FieldKind is fkDataSet by default
  with TDataSetField.Create(Self) do
  begin
    Name := 'TopNested';
    FieldName := 'NestedDataSet';
    DataSet := TopLevelMemTable;
  end;

  //MidLevelMemTable
  MidLevelMemTable.DataSetField := TDataSetField(FindComponent('TopNested'));
  with TIntegerField.Create(Self) do
  begin
    Name := 'MidID';
    FieldKind := fkData;
    FieldName := 'MidID';
    DataSet := MidLevelMemTable;
    Required := True;
  end;
  with TStringField.Create(Self) do
  begin
    Name := 'MidName';
    FieldKind := fkData;
    FieldName := 'MidName';
    DataSet := MidLevelMemTable;
    Size := 30;;
  end;
  with TDataSetField.Create(Self) do
  begin
    Name := 'MidNested';
    FieldName := 'NestedNestedDataSet';
    DataSet := MidLevelMemTable;
  end;

  //Third Level
  ThirdLevelMemTable.DataSetField :=
    TDataSetField(FindComponent('MidNested'));
  with TIntegerField.Create(Self) do
  begin
    Name := 'ThirdID';
    FieldKind := fkData;
    FieldName := 'ThirdID';
    DataSet := ThirdLevelMemTable;
    Required := True;
  end;
  with TStringField.Create(Self) do
  begin
    Name := 'ThirdName';
    FieldKind := fkData;
    FieldName := 'ThirdName';
    DataSet := ThirdLevelMemTable;
    Size := 25;
  end;
  with TBooleanField.Create(Self) do
  begin
    Name := 'ThirdActive';
    FieldKind := fkData;
    FieldName := 'ThirdActive';
    DataSet := ThirdLevelMemTable;
  end;
  //Create the ClientDataSet and its nested datasets
  TopLevelMemTable.Active := True;;
  //Configure the DataSources
  DataSource1.DataSet := TopLevelMemTable;
  (**)
  DataSource2.DataSet := MidLevelMemTable;
  Datasource3.DataSet := ThirdLevelMemTable;
  (**)
  (*
  DataSource2.DataSet :=
    DataSetField(FindComponent('TopNested')).NestedDataSet;
  Datasource3.DataSet :=
    TDataSetField(FindComponent('MidNested')).NestedDataSet;
  (**)
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  dir := ExtractFileDir( ParamStr(0) );
end;

end.
