(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit datamoduleu;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, FireDAC.UI.Intf,
  FireDAC.FMXUI.Async, FireDAC.Comp.UI, Data.DB, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Phys,
  FireDAC.FMXUI.Wait, FireDAC.Phys.IBBase, FireDAC.Phys.IB;

type
  TDataModule2 = class(TDataModule)
    FDQuery1: TFDQuery;
    FDGUIxAsyncExecuteDialog1: TFDGUIxAsyncExecuteDialog;
    FDManager1: TFDManager;
    FDPhysIBDriverLink1: TFDPhysIBDriverLink;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDMemTable1: TFDMemTable;
    FDConnection1: TFDConnection;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  DataModule2: TDataModule2;

implementation

{%CLASSGROUP 'FMX.Controls.TControl'}

uses DataPaths;

{$R *.dfm}

procedure TDataModule2.DataModuleCreate(Sender: TObject);
var
  Params: TStrings;
begin
  Params := TStringList.Create;
// This is what it would look like manually
//    Params.Add('Database=C:\ProgramData\Embarcadero\InterBase\' +
//                         'gds_db\examples\database\employee.gdb');
//    Params.Add('User_Name=sysdba');
//    Params.Add('Server=127.0.0.1');
//    Params.Add('DriverID=IB');
  try
    // and here is how it is using the DataPaths unit
    Params.AddStrings(DataPaths.ConnectionParams);
    FDManager.AddConnectionDef('IB_EMPLOYEE', 'IB', Params);
  finally
    Params.Free;
  end;
  FDConnection1.ConnectionDefName := 'IB_EMPLOYEE';
  try
    FDConnection1.Open();
  except
    on Exception do
    begin
      raise Exception.Create('Invalid configuration information. Please update the ' +
                             'DataPaths file with correct database location ' +
                             'information. See the connection information for the ' +
                             'employee.gdb database in the InterBase node of the ' +
                             'data explorer for the database location');
    end;
  end;
end;


procedure TDataModule2.DataModuleDestroy(Sender: TObject);
begin
  FDManager.Close;
end;

end.
