(* Copyright (c) 2017. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: FireDAC"
     by Cary Jensen

   ISBN-13: 978-1546391272 
   ISBN-10: 1546391274

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: FireDAC"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/FireDACBook
*)
unit mainformu;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants, 
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  System.Rtti, FMX.Layouts, Fmx.Bind.Navigator, FMX.Grid, Data.Bind.EngExt,
  Fmx.Bind.DBEngExt, Fmx.Bind.Grid, System.Bindings.Outputs, Fmx.Bind.Editors,
  Data.Bind.Components, Data.Bind.Grid, Data.DB, Data.Bind.DBScope, FireDAC.Stan.Intf,
  Data.Bind.Controls, FireDAC.Stan.Option, FireDAC.Stan.Param,
  FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf,
  FireDAC.Stan.Async, FireDAC.DApt, FireDAC.Comp.DataSet, FireDAC.Comp.Client;

type
  TForm1 = class(TForm)
    Grid1: TGrid;
    BindNavigator1: TBindNavigator;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Label1: TLabel;
    BindSourceDB1: TBindSourceDB;
    BindingsList1: TBindingsList;
    LinkGridToDataSourceBindSourceDB1: TLinkGridToDataSource;
    ThreadCountLbl: TLabel;
    Button5: TButton;
    FDQuery1: TFDQuery;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure QueryAfterOpen(DataSet: TDataSet);
    procedure ThreadDone(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

uses DataModuleu, QueryThreadu;

procedure TForm1.Button1Click(Sender: TObject);
begin
  BindSourceDB1.DataSet.Close;
  BindSourceDB1.DataSet.DisableControls;
  try
    BindSourceDB1.DataSet := DataModule2.FDQuery1;
    DataModule2.FDQuery1.AfterOpen := nil;
    DataModule2.FDQuery1.ResourceOptions.CmdExecMode := amBlocking;
    DataModule2.FDQuery1.Open;
  finally
    BindSourceDB1.DataSet := DataModule2.FDQuery1;
    BindSourceDB1.DataSet.EnableControls;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  BindSourceDB1.DataSet.Close;
  BindSourceDB1.DataSet.DisableControls;
  try
    BindSourceDB1.DataSet := DataModule2.FDQuery1;;
    DataModule2.FDQuery1.AfterOpen := nil;
    DataModule2.FDQuery1.ResourceOptions.CmdExecMode := amCancelDialog;
    DataModule2.FDQuery1.Open;
  finally
    BindSourceDB1.DataSet := DataModule2.FDQuery1;
    BindSourceDB1.DataSet.EnableControls;
  end;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  BindSourceDB1.DataSet.Close;
  BindSourceDB1.DataSet.DisableControls;
  BindSourceDB1.DataSet := DataModule2.FDQuery1;
  DataModule2.FDQuery1.AfterOpen := QueryAfterOpen;
  DataModule2.FDQuery1.ResourceOptions.CmdExecMode := amAsync;
  DataModule2.FDQuery1.Open;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  QueryThread: TQueryThread;
begin
  BindSourceDB1.DataSet := nil;
  QueryThread := TQueryThread.Create(DataModule2.FDQuery1.SQL.Text,
                                     DataModule2.FDMemTable1);
  QueryThread.FreeOnTerminate := True;
  QueryThread.OnTerminate := ThreadDone;
  QueryThread.Start;
end;

procedure TForm1.Button5Click(Sender: TObject);
begin
  BindSourceDB1.DataSet.Close;
  BindSourceDB1.DataSet.DisableControls;
  BindSourceDB1.DataSet := DataModule2.FDQuery1;
  DataModule2.FDQuery1.AfterOpen := QueryAfterOpen;
  DataModule2.FDQuery1.ResourceOptions.CmdExecMode := amNonBlocking;
  DataModule2.FDQuery1.Open;
end;

procedure TForm1.QueryAfterOpen(DataSet: TDataSet);
begin
  BindSourceDB1.DataSet := DataModule2.FDQuery1;
  BindSourceDB1.DataSet.EnableControls;
end;

procedure TForm1.ThreadDone(Sender: TObject);
begin
  BindSourceDB1.DataSet := DataModule2.FDMemTable1;
  ThreadCountLbl.Text := IntToStr(StrToInt(ThreadCountLbl.Text) + 1);
end;

end.
